# -*- coding: utf-8 -*-

# Copyright(C) 2014 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


class BrowserIncorrectPassword(Exception):
    pass


class BrowserForbidden(Exception):
    pass


class BrowserBanned(BrowserIncorrectPassword):
    pass


class BrowserUnavailable(Exception):
    pass


class BrowserQuestion(BrowserIncorrectPassword):
    """
    When raised by a browser,
    """
    def __init__(self, *fields):
        self.fields = fields


class BrowserHTTPNotFound(BrowserUnavailable):
    pass


class BrowserHTTPError(BrowserUnavailable):
    pass


class BrowserHTTPSDowngrade(BrowserUnavailable):
    pass


class BrowserSSLError(BrowserUnavailable):
    pass


class ParseError(Exception):
    pass


class FormFieldConversionWarning(UserWarning):
    """
    A value has been set to a form's field and has been implicitly converted.
    """


class NoAccountsException(Exception):
    pass


class ModuleInstallError(Exception):
    pass


class ModuleLoadError(Exception):
    def __init__(self, module_name, msg):
        super(ModuleLoadError, self).__init__(msg)
        self.module = module_name


class ActionNeeded(Exception):
    pass


class BrowserPasswordExpired(ActionNeeded):
    pass



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.
from weboob.browser import PagesBrowser, URL
from .pages import ResultsPage, RecipePage, HomePage

from weboob.tools.compat import urlencode

__all__ = ['AllrecipesBrowser']


class AllrecipesBrowser(PagesBrowser):
    BASEURL = 'https://apps.allrecipes.com'
    results = URL('/v1/recipes\?(?P<query>.*)', ResultsPage)
    recipe = URL('/v1/recipes/(?P<_id>.*)/', RecipePage)
    home = URL('http://allrecipes.com', HomePage)

    TOKEN = None

    def fill_token(self):
        self.home.open()
        self.TOKEN = 'Bearer %s' % self.session.cookies.get('ARToken')
        self.session.headers['X-Requested-With'] = 'XMLHttpRequest'
        self.session.headers['Authorization'] = self.TOKEN

    def iter_recipes(self, pattern):
        query = {'query': pattern,
                 'page': 1,
                 'pagesize': 20,
                 'sort': 're'}

        if not self.TOKEN:
            self.fill_token()

        return self.results.go(query=urlencode(query)).iter_recipes()

    def get_recipe(self, _id, obj=None):
        if not self.TOKEN:
            self.fill_token()

        recipe = self.recipe.go(_id=_id).get_recipe(obj=obj)
        comments = list(self.page.get_comments())
        if comments:
            recipe.comments = comments
        return recipe



"browser for inrocks.fr website"
# -*- coding: utf-8 -*-

# Copyright(C) 2011  Julien Hebert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from .pages import ArticlePage
from weboob.browser.browsers import AbstractBrowser
from weboob.browser.url import URL


class NewspaperInrocksBrowser(AbstractBrowser):
    "NewspaperInrocksBrowser class"
    PARENT = 'genericnewspaper'
    BASEURL = 'http://www.lesinrocks.com'

    article = URL('/\?p=.+',
                  '/\d{4}/\d{2}/\d{2}/actualite/.*',
                  'http://blogs.lesinrocks.com/.*',
                  '/.*',
                  ArticlePage)

    def __init__(self, *args, **kwargs):
        self.weboob = kwargs['weboob']
        super(NewspaperInrocksBrowser, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword
from weboob.capabilities.messages import Message
from .pages import LoginPage, LoginErrorPage, ThreadPage, Tweet, TrendsPage,\
    TimelinePage, HomeTimelinePage, SearchTimelinePage, SearchPage


__all__ = ['TwitterBrowser']


class TwitterBrowser(LoginBrowser):
    BASEURL = 'https://twitter.com/'

    authenticity_token = None

    thread_page = URL(u'(?P<user>.+)/status/(?P<_id>.+)', ThreadPage)
    login_error = URL(u'login/error.+', LoginErrorPage)
    tweet = URL(u'i/tweet/create', Tweet)
    trends = URL(u'i/trends\?pc=true&show_context=false&src=search-home&k=(?P<token>.*)', TrendsPage)
    search = URL(u'i/search/timeline', SearchTimelinePage)
    search_page = URL(u'search\?q=(?P<pattern>.+)&src=sprv',
                      u'search-home', SearchPage)
    profil = URL(u'i/profiles/show/(?P<path>.+)/timeline/tweets', HomeTimelinePage)
    timeline = URL(u'i/timeline', TimelinePage)
    login = URL(u'', LoginPage)

    def do_login(self):
        self.login.stay_or_go()

        if not self.authenticity_token:
            self.authenticity_token = self.page.login(self.username, self.password)

        if not self.page.logged or self.login_error.is_here():
            raise BrowserIncorrectPassword()

    @need_login
    def get_me(self):
        return self.login.stay_or_go().get_me()

    @need_login
    def iter_threads(self):
        return self.timeline.go().iter_threads()

    def get_trendy_subjects(self):
        if self.username:
            return self.get_logged_trendy_subject()
        else:
            return self.trends.open(token="").get_trendy_subjects()

    def get_logged_trendy_subject(self):
        if not self.authenticity_token:
            self.do_login()

        trends_token = self.search_page.open().get_trends_token()
        return self.trends.open(token=trends_token).get_trendy_subjects()

    @need_login
    def post(self, thread, message):
        datas = {'place_id': '',
                 'tagged_users': ''}
        datas['authenticity_token'] = self.authenticity_token
        datas['status'] = message
        if thread:
            datas['in_reply_to_status_id'] = thread.id.split('#')[-1]

        self.tweet.open(data=datas)

    def get_thread(self, _id, thread=None, seen=None):
        splitted_id = _id.split('#')

        if not thread:
            thread = self.thread_page.go(_id=splitted_id[1].split('.')[-1], user=splitted_id[0]).get_thread(obj=thread)

        title_content = thread.title.split('\n\t')[-1]

        thread.root = Message(thread=thread,
                              id=splitted_id[1].split('.')[-1],
                              title=title_content[:50] if len(title_content) > 50 else title_content,
                              sender=splitted_id[0],
                              receivers=None,
                              date=thread.date,
                              parent=thread.root,
                              content=title_content,
                              signature=u'',
                              children=[]
                              )

        if seen and (_id not in seen):
            thread.root.flags = Message.IS_UNREAD

        comments = self.thread_page.stay_or_go(_id=splitted_id[1].split('.')[-1], user=splitted_id[0]).iter_comments()
        for comment in comments:
            comment.thread = thread
            comment.parent = thread.root
            if seen and comment.id not in seen.keys():
                comment.flags = Message.IS_UNREAD

            thread.root.children.append(comment)

        return thread

    def get_tweets_from_profil(self, path):
        return self.profil.go(path=path).iter_threads()

    def get_tweets_from_hashtag(self, path):
        return self.get_tweets_from_search(u'#%s' % path if not path.startswith('#') else path)

    def get_tweets_from_search(self, path):
        min_position = self.search_page.go(pattern=path).get_min_position()
        params = {'q': "%s" % path,
                  'src': 'sprv'}

        return self.search.go(params=params).iter_threads(params=params, min_position=min_position)



# -*- coding: utf-8 -*-

# Copyright(C) 2012 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.test import BackendTest


class BanquePopulaireTest(BackendTest):
    MODULE = 'banquepopulaire'

    def test_banquepop(self):
        l = list(self.backend.iter_accounts())
        if len(l) > 0:
            a = l[0]
            list(self.backend.iter_history(a))



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from base64 import b64decode, b64encode
from collections import OrderedDict
import math
from os import urandom

from Cryptodome.Cipher import AES
from Cryptodome.Protocol.KDF import PBKDF2
from Cryptodome.Hash import SHA256
from Cryptodome.Hash import HMAC


def log2(n):
    return math.log(n, 2)


def encrypt(plaintext):
    iv = urandom(16)
    salt = urandom(8)
    iterations = 1000
    ks = 128
    ts = 64

    hash_func = lambda k, s: HMAC.new(k, s, SHA256).digest()
    password = b64encode(urandom(32))
    key = PBKDF2(password, salt=salt, count=iterations, prf=hash_func)

    smalliv = trunc_iv(iv, plaintext, 0)

    cipher = AES.new(key, mode=AES.MODE_CCM, nonce=smalliv, mac_len=ts // 8)
    ciphertext = b''.join(cipher.encrypt_and_digest(plaintext))

    # OrderedDict because 0bin is a piece of shit requiring "iv" as the first key
    return password.decode('ascii'), OrderedDict([
        ('iv', b64encode(iv).decode('ascii')),
        ('v', 1),
        ('iter', iterations),
        ('ks', ks),
        ('ts', ts),
        ('mode', 'ccm'),
        ('adata', ''),
        ('cipher', 'aes'),
        ('salt', b64encode(salt).decode('ascii')),
        ('ct', b64encode(ciphertext).decode('ascii')),
    ])


def trunc_iv(iv, t, tl):
    ol = len(t) - tl
    if ol <= 0:
        oll = 2
    else:
        oll = int(max(2, math.ceil(log2(ol) / 8.)))
    assert oll <= 4
    if oll < 15 - len(iv):
        ivl = len(iv)
    else:
        ivl = 15 - oll
    iv = iv[:ivl]
    return iv


def decrypt(secretkey, params):
    iv = b64decode(params['iv'])
    salt = b64decode(params['salt'])
    #~ keylen = params.get('ks', 128) // 8 # FIXME use somewhere?
    taglen = params.get('ts', 64) // 8
    iterations = params.get('iter', 1000)
    data = b64decode(params['ct'])
    ciphertext = data[:-taglen]
    tag = data[-taglen:]

    if params.get('adata'):
        raise NotImplementedError('authenticated data support is not implemented')

    iv = trunc_iv(iv, ciphertext, taglen)

    hash_func = lambda k, s: HMAC.new(k, s, SHA256).digest()
    key = PBKDF2(secretkey, salt=salt, count=iterations, prf=hash_func)

    mode_str = params.get('mode', 'ccm')
    mode = dict(ccm=AES.MODE_CCM)[mode_str]
    if mode_str == 'ccm':
        cipher = AES.new(key, mode=AES.MODE_CCM, nonce=iv, mac_len=taglen)
    else:
        cipher = AES.new(key, mode=mode, iv=iv)
    decrypted = cipher.decrypt_and_verify(ciphertext, tag)
    return decrypted



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from datetime import datetime, date

from weboob.tools.compat import basestring, unicode

from .base import Capability, BaseObject, Field, FloatField, \
                  StringField, UserError, NotLoaded
from .date import DateField

__all__ = ['Forecast', 'Current', 'City', 'CityNotFound', 'Temperature', 'CapWeather']


class Temperature(BaseObject):

    value =      FloatField('Temperature value')
    unit =       StringField('Input unit')

    def __init__(self, value=NotLoaded, unit = u'', url=None):
        super(Temperature, self).__init__(unicode(value), url)
        self.value = value
        if unit not in [u'C', u'F']:
            unit = u''
        self.unit = unit

    def asfahrenheit(self):
        if not self.unit:
            return u'%s' % int(round(self.value))
        elif self.unit == 'F':
            return u'%s °F' % int(round(self.value))
        else:
            return u'%s °F' % int(round((self.value * 9.0 / 5.0) + 32))

    def ascelsius(self):
        if not self.unit:
            return u'%s' % int(round(self.value))
        elif self.unit == 'C':
            return u'%s °C' % int(round(self.value))
        else:
            return u'%s °C' % int(round((self.value - 32.0) * 5.0 / 9.0))

    def __repr__(self):
        if self.value is not None and self.unit:
            return '%r %r' % (self.value, self.unit)
        return ''


class Forecast(BaseObject):
    """
    Weather forecast.
    """
    date =      Field('Date for the forecast', datetime, date, basestring)
    low =       Field('Low temperature', Temperature)
    high =      Field('High temperature', Temperature)
    text =      StringField('Comment on forecast')

    def __init__(self, date=NotLoaded, low=None, high=None, text=None, unit=None, url=None):
        super(Forecast, self).__init__(unicode(date), url)
        self.date = date
        self.low = Temperature(low, unit)
        self.high = Temperature(high, unit)
        self.text = text


class Current(BaseObject):
    """
    Current weather.
    """
    date =      DateField('Date of measure')
    text =      StringField('Comment about current weather')
    temp =      Field('Current temperature', Temperature)

    def __init__(self, date=NotLoaded, temp=None, text=None, unit=None, url=None):
        super(Current, self).__init__(unicode(date), url)
        self.date = date
        self.text = text
        self.temp = Temperature(temp, unit)


class City(BaseObject):
    """
    City where to find weather.
    """
    name =      StringField('Name of city')

    def __init__(self, id='', name=None, url=None):
        super(City, self).__init__(id, url)
        self.name = name


class CityNotFound(UserError):
    """
    Raised when a city is not found.
    """


class CapWeather(Capability):
    """
    Capability for weather websites.
    """

    def iter_city_search(self, pattern):
        """
        Look for a city.

        :param pattern: pattern to search
        :type pattern: str
        :rtype: iter[:class:`City`]
        """
        raise NotImplementedError()

    def get_current(self, city_id):
        """
        Get current weather.

        :param city_id: ID of the city
        :rtype: :class:`Current`
        """
        raise NotImplementedError()

    def iter_forecast(self, city_id):
        """
        Iter forecasts of a city.

        :param city_id: ID of the city
        :rtype: iter[:class:`Forecast`]
        """
        raise NotImplementedError()



# * -*- coding: utf-8 -*-

# Copyright(C) 2013  Thomas Lecavelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import PagesBrowser, URL
from .pages import LivePage, StreamsPage

__all__ = ['NectarineBrowser']


class NectarineBrowser(PagesBrowser):
    BASEURL = 'https://www.scenemusic.net'

    streams = URL(r'/demovibes/xml/streams/', StreamsPage)
    live = URL(r'/demovibes/xml/queue/', LivePage)

    def home(self):
        self.location('/demovibes/xml/streams/')

        assert self.streams.is_here()

    def iter_radios_list(self):
        self.location('/demovibes/xml/streams/')

        assert self.streams.is_here()
        return self.page.iter_radios_list()

    def get_current_emission(self):
        self.location('/demovibes/xml/queue/')
        assert self.live.is_here()
        return self.page.get_current_emission()



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import PagesBrowser, URL
from weboob.capabilities.housing import Query, TypeNotSupported
from .pages import CitiesPage, SearchPage, HousingPage, PhonePage


class LogicimmoBrowser(PagesBrowser):
    BASEURL = 'http://www.logic-immo.com/'

    city = URL('asset/t9/getLocalityT9.php\?site=fr&lang=fr&json=%22(?P<pattern>.*)%22',
               CitiesPage)
    search = URL('(?P<type>location-immobilier|vente-immobilier|recherche-colocation)-(?P<cities>.*)/options/(?P<options>.*)', SearchPage)
    housing = URL('detail-(?P<_id>.*).htm', HousingPage)
    phone = URL('(?P<urlcontact>.*)', PhonePage)

    TYPES = {Query.TYPE_RENT: 'location-immobilier',
             Query.TYPE_SALE: 'vente-immobilier',
             Query.TYPE_SHARING: 'recherche-colocation'}

    RET = {Query.HOUSE_TYPES.HOUSE: '2',
           Query.HOUSE_TYPES.APART: '1',
           Query.HOUSE_TYPES.LAND: '3',
           Query.HOUSE_TYPES.PARKING: '10',
           Query.HOUSE_TYPES.OTHER: '14'}

    def get_cities(self, pattern):
        if pattern:
            return self.city.go(pattern=pattern).get_cities()

    def search_housings(self, type, cities, nb_rooms, area_min, area_max, cost_min, cost_max, house_types):

        if type not in self.TYPES:
            raise TypeNotSupported()

        options = []

        ret = []
        for house_type in house_types:
            if house_type in self.RET:
                ret.append(self.RET.get(house_type))

        if len(ret):
            options.append('groupprptypesids=%s' % ','.join(ret))

        options.append('pricemin=%s' % (cost_min if cost_min else '0'))

        if cost_max:
            options.append('pricemax=%s' % cost_max)

        options.append('areamin=%s' % (area_min if area_min else '0'))

        if area_max:
            options.append('areamax=%s' % area_max)

        if nb_rooms:
            if type == Query.TYPE_SHARING:
                options.append('nbbedrooms=%s' % ','.join([str(i) for i in range(nb_rooms, 7)]))
            else:
                options.append('nbrooms=%s' % ','.join([str(i) for i in range(nb_rooms, 7)]))

        self.search.go(type=self.TYPES.get(type, 'location-immobilier'),
                       cities=cities,
                       options='/'.join(options))

        if type == Query.TYPE_SHARING:
            return self.page.iter_sharing()

        return self.page.iter_housings()

    def get_housing(self, _id, housing=None):
        return self.housing.go(_id=_id).get_housing(obj=housing)

    def get_phone(self, _id):
        if _id.startswith('location') or _id.startswith('vente'):
            urlcontact, params = self.housing.stay_or_go(_id=_id).get_phone_url_datas()
            return self.phone.go(urlcontact=urlcontact, params=params).get_phone()



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import PagesBrowser, URL

from .pages import ListStationsPage, InfoStationPage


__all__ = ['VlilleBrowser']


class VlilleBrowser(PagesBrowser):

    BASEURL = 'http://www.vlille.fr'
    list_page = URL('/stations/les-stations-vlille.aspx', ListStationsPage)
    info_page = URL('/stations/xml-station.aspx\?borne=(?P<idgauge>.*)', InfoStationPage)

    def get_station_list(self):
        return self.list_page.go().get_station_list()

    def get_station_infos(self, gauge):
        return self.info_page.go(idgauge=gauge.id).get_station_infos(obj=gauge)



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Edouard Lambert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword

from .pages import (
    LoginPage, AccountsPage, AMFHSBCPage, AMFAmundiPage, AMFSGPage, HistoryPage,
    ErrorPage, LyxorfcpePage, EcofiPage,
)


class S2eBrowser(LoginBrowser):
    login = URL('/portal/salarie-(?P<slug>\w+)/authentification',
                '/portal/j_security_check', LoginPage)
    accounts = URL('/portal/salarie-(?P<slug>\w+)/monepargne/mesavoirs\?language=(?P<lang>)',
                   '/portal/salarie-(?P<slug>\w+)/monepargne/mesavoirs', AccountsPage)
    amfcode_hsbc = URL('https://www.assetmanagement.hsbc.com/feedRequest', AMFHSBCPage)
    amfcode_amundi = URL('https://www.amundi-ee.com/entr/product', AMFAmundiPage)
    amfcode_sg = URL('http://sggestion-ede.com/product', AMFSGPage)
    isincode_ecofi = URL(r'http://www.ecofi.fr/fr/fonds/.*#yes\?bypass=clientprive', EcofiPage)
    lyxorfcpe = URL('http://www.lyxorfcpe.com/part', LyxorfcpePage)
    history = URL('/portal/salarie-(?P<slug>\w+)/operations/consulteroperations', HistoryPage)
    error = URL('/maintenance/HSBC/', ErrorPage)

    def __init__(self, *args, **kwargs):
        self.secret = None
        if 'secret' in kwargs:
            self.secret = kwargs['secret']
            del kwargs['secret']
        super(S2eBrowser, self).__init__(*args, **kwargs)
        self.cache = {}
        self.cache['invs'] = {}
        self.cache['pockets'] = {}
        self.cache['details'] = {}

    def do_login(self):
        self.login.go(slug=self.SLUG).login(self.username, self.password, self.secret)

        if self.login.is_here():
            error = self.page.get_error()
            raise BrowserIncorrectPassword(error)

    @need_login
    def iter_accounts(self):
        if 'accs' not in self.cache.keys():
            self.accounts.stay_or_go(slug=self.SLUG, lang=self.LANG)
            # weird wrongpass
            if not self.accounts.is_here():
                raise BrowserIncorrectPassword()
            multi = self.page.get_multi()
            if len(multi):
                # Handle multi entreprise accounts
                accs = []
                for id in multi:
                    self.page.go_multi(id)
                    for a in self.accounts.go(slug=self.SLUG).iter_accounts():
                        a._multi = id
                        accs.append(a)
            else:
                accs = [a for a in self.page.iter_accounts()]
            self.cache['accs'] = accs
        return self.cache['accs']

    @need_login
    def iter_investment(self, account):
        if account.id not in self.cache['invs']:
            self.accounts.stay_or_go(slug=self.SLUG)
            # Handle multi entreprise accounts
            if hasattr(account, '_multi'):
                self.page.go_multi(account._multi)
                self.accounts.go(slug=self.SLUG)
            # Select account
            self.page.get_investment_pages(account.id)
            invs = [i for i in self.page.iter_investment()]
            # Get page with quantity
            self.page.get_investment_pages(account.id, valuation=False)
            self.cache['invs'][account.id] = self.page.update_invs_quantity(invs)
        return self.cache['invs'][account.id]

    @need_login
    def iter_pocket(self, account):
        if account.id not in self.cache['pockets']:
            self.iter_investment(account)
            # Select account
            self.page.get_investment_pages(account.id, pocket=True)
            pockets = [p for p in self.page.iter_pocket(accid=account.id)]
            # Get page with quantity
            self.page.get_investment_pages(account.id, valuation=False, pocket=True)
            self.cache['pockets'][account.id] = self.page.update_pockets_quantity(pockets)
        return self.cache['pockets'][account.id]

    @need_login
    def iter_history(self, account):
        self.history.stay_or_go(slug=self.SLUG)
        # Handle multi entreprise accounts
        if hasattr(account, '_multi'):
            self.page.go_multi(account._multi)
            self.history.go(slug=self.SLUG)
        # Get more transactions on each page
        self.page.show_more("50")
        for tr in self.page.iter_history(accid=account.id):
            yield tr
        # Go back to first page
        self.page.go_start()


class EsaliaBrowser(S2eBrowser):
    BASEURL = 'https://salaries.esalia.com'
    SLUG = 'sg'
    LANG = 'fr' # ['fr', 'en']


class CapeasiBrowser(S2eBrowser):
    BASEURL = 'https://www.capeasi.com'
    SLUG = 'axa'
    LANG = 'fr' # ['fr', 'en']


class ErehsbcBrowser(S2eBrowser):
    BASEURL = 'https://epargnant.ere.hsbc.fr'
    SLUG = 'hsbc'
    LANG = 'fr' # ['fr', 'en']


class BnppereBrowser(S2eBrowser):
    BASEURL = 'https://personeo.epargne-retraite-entreprises.bnpparibas.com'
    SLUG = 'bnp'
    LANG = 'fr' # ['fr', 'en']



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.tools.backend import Module, BackendConfig
from weboob.capabilities.paste import CapPaste, BasePaste
from weboob.tools.capabilities.paste import image_mime
from weboob.tools.compat import urljoin
from weboob.tools.value import Value

from .browser import LutimBrowser


__all__ = ['LutimModule']


class LutimModule(Module, CapPaste):
    NAME = 'lutim'
    DESCRIPTION = u'lutim website'
    MAINTAINER = u'Vincent A'
    EMAIL = 'dev@indigo.re'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    BROWSER = LutimBrowser

    CONFIG = BackendConfig(Value('base_url', label='Hoster base URL', default='https://lut.im/'))

    @property
    def base_url(self):
        url = self.config['base_url'].get()
        if not url.endswith('/'):
            url = url + '/'
        return url

    def create_default_browser(self):
        return self.create_browser(self.base_url)

    def can_post(self, contents, title=None, public=None, max_age=None):
        if public:
            return 0
        elif max_age and max_age < 86400:
            return 0 # it cannot be shorter than one day
        elif re.search(r'[^a-zA-Z0-9=+/\s]', contents):
            return 0 # not base64, thus not binary
        else:
            mime = image_mime(contents, ('gif', 'jpeg', 'png'))
            return 20 * int(mime is not None)

    def get_paste(self, url):
        if not url.startswith('http'):
            url = urljoin(self.base_url, url)
        paste = self.new_paste(url)
        self.browser.fetch(paste)
        return paste

    def new_paste(self, _id):
        paste = LutimPaste(_id)
        return paste

    def post_paste(self, paste, max_age):
        return self.browser.post(paste, max_age)


class LutimPaste(BasePaste):
    @classmethod
    def id2url(cls, id):
        return id



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from weboob.browser import PagesBrowser, URL
from weboob.tools.json import json

from .pages import ConcertListPage, ConcertPage, NoEvent


__all__ = ['SueurDeMetalBrowser']


class SueurDeMetalBrowser(PagesBrowser):
    BASEURL = 'http://www.sueurdemetal.com'

    concerts = URL(r'/func/listconcert-timeline.php', ConcertListPage)
    concert = URL(r'/func/funcGetEvent.php', ConcertPage)

    def __init__(self, *args, **kwargs):
        super(SueurDeMetalBrowser, self).__init__(*args, **kwargs)
        self.cities = {}

    def jlocation(self, *args, **kwargs):
        kwargs.setdefault('headers', {})['Content-Type'] = 'application/json;charset=utf-8'
        kwargs['data'] = json.dumps(kwargs['data'])
        return super(SueurDeMetalBrowser, self).location(*args, **kwargs)

    def search_city(self, city):
        self.jlocation(self.concerts.build(), data={
            'date': '00',
            'dep': '00',
            'salle': '00',
            'groupes': '00',
            'ville': city,
        })
        return self.page.iter_concerts()

    def get_concert(self, id):
        self.jlocation(self.concert.build(), data={
            'id': id,
        })
        try:
            return self.page.get_concert()
        except NoEvent:
            return None

    def build_cities(self):
        if self.cities:
           return
        self.deps.go()
        for dept in self.page.get_depts():
            self.jlocation(self.cities.build, data={
                'annee': '00',
                '': '',
            })



#!C:\Python27\python.exe
# -*- coding: utf-8 -*-
#
# windows icon file(.ico) and cursor file(.cur) generator/writer from RGB(a) array
#
#    NOTICE: one image per file *only*, 16x16 or 32x32 or 48x48 or 64x64 *only*
#    Support 24 bit & 32 bit(RGB + alpha)
#
# Copyright 2009 ( http://creativecommons.org/licenses/by-nc-sa/2.5/cn/ )
#
# About Author
#   Blog( http://2maomao.com/blog/ )
#   Website( http://fayaa.com/tool/favicon/ )
#   Contact me via email
#
# References:
#  [1] http://en.wikipedia.org/wiki/Ico_Format
#  [2] http://en.wikipedia.org/wiki/Windows_and_OS/2_bitmap
#  [3] http://support.microsoft.com/kb/83034
#  [4] http://msdn.microsoft.com/en-us/library/ms997538(loband).aspx
#  [5] http://www.martinreddy.net/gfx/2d/BMP.txt
#  [6] http://code.google.com/p/casadebender/source/browse/python/PIL/Win32IconImagePlugin.py
#  [7] http://www.osphp.com.cn/read.php/290.htm


from PIL import Image
import sys
import os

# data: a size*size of (r,g,b,t) tuples, t for tansparency(if True)
#       (r,g,b,a) if bpp is 32
# size: could be 16 or 32 or 48 or 64, other value NOT supported
# bpp: bits per pixel, could *only* be 24 or 32!
# return: an "array" of BYTES which, if write to file, is a size*size ico file


def genico(data, size=16, bpp=24):
    from array import array
    a = array('B')
    #header(ref1&5)
    a.extend((0,0, 1,0, 1,0)) #reserved*2, icon,0, 1 image per-file,0
    #directory(ref1&5&7)
    # image-part length in bytes
    # !hack! AND bits align to 32 bits per line
    # !shit! MSDN says nothing about this
    imglen = 40+size*(size*3 + (size+16)/32*32/8)

    if bpp == 32:
        imglen += size*size #1 more byte for alpha value of each pixel
    a.extend((size,size, 0,0, 1,0, bpp,0)) #w,h, reserved*2, 1plane*2, bpp*2
    a.extend((imglen&0xff,imglen>>8,0,0, 22,0,0,0)) #bitmap-size*4,22B-offset*4
    #image BITMAPINFOHEADER(ref5)
    a.extend((40,0,0,0)) #size of data(contains header)*4
    a.extend((size,0,0,0, size*2,0,0,0))#w*4, (h+h)*4 (!shit hack! XOR+AND)
    a.extend((1,0, bpp,0, 0,0,0,0, 0,0,0,0)) #1 plane*2, 24 bits*2, no compress*4, rawBMPsize(no compress so 0)*4
    a.extend((0,1,0,0, 0,1,0,0)) #horizontal*4/vertical*4 resolution pixels/meter(WTF?)
    a.extend((0,0,0,0, 0,0,0,0)) #colors fully used, all are important
    #!no planes
    #image content(ref1&5), XOR+AND bitmaps
    AND = array('B')
    vand = 0
    vcnt = 0
    #remember that bitmap format is reversed in y-axis
    for x in range(size-1,-1,-1):
        for y in range(0, size):
            b,g,r,t_or_a = data[y*size+x]
            a.extend((r,g,b))
            if bpp == 32:
                a.append(t_or_a)
            vcnt+=1
            vand<<=1
            if (bpp==24 and t_or_a) or (bpp==32 and t_or_a<128):
                vand |= 1
            if vcnt==8:
                AND.append(vand)
                vcnt=0
                vand=0
        #!hack! AND bits align to 32 bits per line, !shit! MSDN says nothing about this
        AND.extend([0] * ((32-size%32)%32/8))
    a.extend(AND)
    return a


# x,y indicate the hotspot position
# simply set the type/hotspot(x&y) after generates the icon
def gencur(data, size=16, bpp=24, x=0, y=0):
    a = genico(data, size, bpp)
    a[3], a[10], a[12] = 2, x, y
    return a


#C:\Python27\Lib\site-packages\weboob-0.g-py2.7.egg\share\icons\hicolor\64x64\apps

if __name__ == "__main__":
    if len(sys.argv) == 2:
        png_file = sys.argv[1]

        f, e = os.path.splitext(png_file)
        ico_file = f + ".ico"

        im = Image.open(r"%s" % png_file)
        wh = 64

        rgba = im.convert("RGBA")

        data = []
        for i in range(wh):
            for j in range(wh):
                data.append(rgba.getpixel((i,j)))

        icoflow = genico(data, wh, 32)
        _file = open(ico_file, "wb")
        icoflow.tofile(_file)
        _file.close()



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.deprecated.browser import Browser

from .pages.board import BoardPage


class FourChan(Browser):
    DOMAIN = 'boards.4chan.org'
    PAGES = {
        'http://boards.4chan.org/\w+/': BoardPage,
        'http://boards.4chan.org/\w+/res/\d+': BoardPage,
        }

    def is_logged(self):
        return True

    def get_threads(self, board):
        self.location('http://boards.4chan.org/%s/' % board)
        return self.page.articles

    def get_thread(self, board, id):
        self.location('http://boards.4chan.org/%s/res/%d' % (board, long(id)))
        assert len(self.page.articles) == 1
        return self.page.articles[0]



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2013  Romain Bignon, Pierre Mazière
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from decimal import Decimal
from functools import wraps
import re

from weboob.capabilities.bank import CapBankTransferAddRecipient, AccountNotFound, \
                                     RecipientNotFound, TransferError, Account
from weboob.capabilities.bill import CapDocument, Subscription, SubscriptionNotFound, \
                                     Document, DocumentNotFound
from weboob.capabilities.contact import CapContact
from weboob.capabilities.profile import CapProfile
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.capabilities.bank.transactions import sorted_transactions
from weboob.tools.value import ValueBackendPassword, Value
from weboob.capabilities.base import find_object, NotAvailable

from .browser import LCLBrowser, LCLProBrowser, ELCLBrowser
from .enterprise.browser import LCLEnterpriseBrowser, LCLEspaceProBrowser


__all__ = ['LCLModule']


def only_for_websites(*cfg):
    def decorator(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            if self.config['website'].get() not in cfg:
                raise NotImplementedError()

            return func(self, *args, **kwargs)

        return wrapper
    return decorator


class LCLModule(Module, CapBankTransferAddRecipient, CapContact, CapProfile, CapDocument):
    NAME = 'lcl'
    MAINTAINER = u'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    VERSION = '1.3'
    DESCRIPTION = u'LCL'
    LICENSE = 'AGPLv3+'
    CONFIG = BackendConfig(ValueBackendPassword('login',    label='Identifiant', masked=False),
                           ValueBackendPassword('password', label='Code personnel'),
                           Value('website', label='Type de compte', default='par',
                                 choices={'par': 'Particuliers',
                                          'pro': 'Professionnels',
                                          'ent': 'Entreprises',
                                          'esp': 'Espace Pro',
                                          'elcl': 'e.LCL'}))
    BROWSER = LCLBrowser

    def create_default_browser(self):
        # assume all `website` option choices are defined here
        browsers = {'par': LCLBrowser,
                    'pro': LCLProBrowser,
                    'elcl': ELCLBrowser,
                    'ent': LCLEnterpriseBrowser,
                    'esp': LCLEspaceProBrowser}

        website_value = self.config['website']
        self.BROWSER = browsers.get(website_value.get(),
                                    browsers[website_value.default])

        return self.create_browser(self.config['login'].get(),
                                   self.config['password'].get())

    def iter_accounts(self):
        return self.browser.get_accounts_list()

    def get_account(self, _id):
        return find_object(self.browser.get_accounts_list(), id=_id, error=AccountNotFound)

    def iter_coming(self, account):
        transactions = sorted_transactions(self.browser.get_cb_operations(account))
        return transactions

    def iter_history(self, account):
        transactions = sorted_transactions(self.browser.get_history(account))
        return transactions

    def iter_investment(self, account):
        return self.browser.get_investment(account)

    @only_for_websites('par', 'pro', 'elcl')
    def iter_transfer_recipients(self, origin_account):
        if not isinstance(origin_account, Account):
            origin_account = find_object(self.iter_accounts(), id=origin_account, error=AccountNotFound)
        return self.browser.iter_recipients(origin_account)

    @only_for_websites('par', 'pro')
    def new_recipient(self, recipient, **params):
        # Recipient label has max 15 alphanumrical chars.
        recipient.label = ' '.join(w for w in re.sub('[^0-9a-zA-Z ]+', '', recipient.label).split())[:15]
        return self.browser.new_recipient(recipient, **params)

    @only_for_websites('par', 'pro')
    def init_transfer(self, transfer, **params):
        # There is a check on the website, transfer can't be done with too long reason.
        if transfer.label:
            transfer.label = transfer.label[:30]

        self.logger.info('Going to do a new transfer')
        if transfer.account_iban:
            account = find_object(self.iter_accounts(), iban=transfer.account_iban, error=AccountNotFound)
        else:
            account = find_object(self.iter_accounts(), id=transfer.account_id, error=AccountNotFound)

        if transfer.recipient_iban:
            recipient = find_object(self.iter_transfer_recipients(account.id), iban=transfer.recipient_iban, error=RecipientNotFound)
        else:
            recipient = find_object(self.iter_transfer_recipients(account.id), id=transfer.recipient_id, error=RecipientNotFound)

        try:
            # quantize to show 2 decimals.
            amount = Decimal(transfer.amount).quantize(Decimal(10) ** -2)
        except (AssertionError, ValueError):
            raise TransferError('something went wrong')

        return self.browser.init_transfer(account, recipient, amount, transfer.label)

    def execute_transfer(self, transfer, **params):
        return self.browser.execute_transfer(transfer)

    @only_for_websites('par')
    def iter_contacts(self):
        return self.browser.get_advisor()

    def get_profile(self):
        if not hasattr(self.browser, 'get_profile'):
            raise NotImplementedError()
        return self.browser.get_profile()

    @only_for_websites('par')
    def get_document(self, _id):
        return find_object(self.iter_documents(None), id=_id, error=DocumentNotFound)

    @only_for_websites('par')
    def get_subscription(self, _id):
        return find_object(self.iter_subscription(), id=_id, error=SubscriptionNotFound)

    @only_for_websites('par')
    def iter_bills(self, subscription):
        return self.iter_documents(None)

    @only_for_websites('par')
    def iter_documents(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)

        return self.browser.iter_documents(subscription)

    @only_for_websites('par')
    def iter_subscription(self):
        return self.browser.iter_subscriptions()

    @only_for_websites('par')
    def download_document(self, document):
        if not isinstance(document, Document):
            document = self.get_document(document)
        if document.url is NotAvailable:
            return

        return self.browser.open(document.url).content

    def iter_resources(self, objs, split_path):
        if Account in objs:
            self._restrict_level(split_path)
            return self.iter_accounts()
        if Subscription in objs:
            self._restrict_level(split_path)
            return self.iter_subscription()



# -*- coding: utf-8 -*-

# Copyright(C) 2011-2012  Romain Bignon, Laurent Bachelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.video import BaseVideo
from weboob.tools.test import BackendTest


class PluzzTest(BackendTest):
    MODULE = 'francetelevisions'

    def test_search(self):
        # If the test fails, it might be good news!
        l = list(self.backend.search_videos("20h"))
        self.assertTrue(len(l) > 0)
        v = l[0]
        v = self.backend.fillobj(v, ('url',)) or v
        self.assertTrue(v.url, 'URL for video "%s" not found: %s' % (v.id, v.url))

    def test_categories(self):
        cat = list(self.backend.iter_resources([BaseVideo], []))
        self.assertTrue(len(cat) > 0)
        for c in cat:
            if c.split_path[-1].startswith('vid_'):
                videos = list(self.backend.iter_resources([BaseVideo], c.split_path))
                self.assertTrue(len(videos) > 0)
                v = videos[0]
                v = self.backend.fillobj(v, ('url',)) or v
                self.assertTrue(v.url, 'URL for video "%s" not found: %s' % (v.id, v.url))
                return



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Matthieu Weber
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from dateutil.parser import parse as parse_date

from weboob.browser.pages import JsonPage
from weboob.capabilities.parcel import Parcel, Event, ParcelNotFound

STATUSES = {
    1: Parcel.STATUS_PLANNED,
    2: Parcel.STATUS_IN_TRANSIT,
    3: Parcel.STATUS_IN_TRANSIT,
    4: Parcel.STATUS_IN_TRANSIT,
    5: Parcel.STATUS_ARRIVED,
}


class SearchPage(JsonPage):
    def build_doc(self, text):
        from weboob.tools.json import json
        return json.loads(text[1:-1])

    def get_info(self, _id):
        result_id = self.doc.get("TrackingStatusJSON", {}).get("shipmentInfo", {}).get("parcelNumber", None)
        if not result_id:
            raise ParcelNotFound("No such ID: %s" % _id)
        if not _id.startswith(result_id):
            raise ParcelNotFound("ID mismatch: expecting %s, got %s" % (_id, result_id))

        p = Parcel(_id)
        events = self.doc.get("TrackingStatusJSON", {}).get("statusInfos", [])
        p.history = [self.build_event(i, data) for i, data in enumerate(events)]
        p.status = self.guess_status(
            self.doc.get("TrackingStatusJSON", {}).
            get("shipmentInfo", {}).
            get("deliveryStatus"))
        p.info = p.history[-1].activity
        return p

    def guess_status(self, status_code):
        return STATUSES.get(status_code, Parcel.STATUS_UNKNOWN)

    def build_event(self, index, data):
        event = Event(index)
        date = "%s %s" % (data["date"], data["time"])
        event.date = parse_date(date, dayfirst=False)
        event.location = unicode(data["city"])
        event.activity = unicode(", ".join([_["label"] for _ in data["contents"]]))
        return event



# -*- coding: utf-8 -*-

# Copyright(C) 2015 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.geolocip import CapGeolocIp, IpLocation
from weboob.tools.backend import Module
from weboob.browser.browsers import Browser
from weboob.tools.json import json


__all__ = ['IpapiModule']


class IpapiModule(Module, CapGeolocIp):
    NAME = 'ipapi'
    MAINTAINER = u'Julien Veyssier'
    EMAIL = 'julien.veyssier@aiur.fr'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'
    DESCRIPTION = u"IP-API Geolocation API"
    BROWSER = Browser

    def get_location(self, ipaddr):
        res = self.browser.location('http://ip-api.com/json/%s' % ipaddr.encode('utf-8'))
        jres = json.loads(res.text)

        if "status" in jres and jres["status"] == "fail":
            raise Exception("IPAPI failure : %s" % jres["message"])

        iploc = IpLocation(ipaddr)
        iploc.city = u'%s'%jres['city']
        iploc.region = u'%s'%jres['regionName']
        iploc.zipcode = u'%s'%jres['zip']
        iploc.country = u'%s'%jres['country']
        if jres['lat'] != '':
            iploc.lt = float(jres['lat'])
        else:
            iploc.lt = 0.0
        if jres['lon'] != '':
            iploc.lg = float(jres['lon'])
        else:
            iploc.lg = 0.0
        #iploc.host = 'NA'
        #iploc.tld = 'NA'
        if 'isp' in jres:
            iploc.isp = u'%s'%jres['isp']

        return iploc



# -*- coding: utf-8 -*-

# Copyright(C) 2013-2015      Christophe Lampin
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.bill import CapDocument, SubscriptionNotFound, DocumentNotFound, Subscription, Bill
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import ValueBackendPassword
from .browser import AmeliProBrowser

__all__ = ['AmeliProModule']

class AmeliProModule(Module, CapDocument):
    NAME = 'amelipro'
    DESCRIPTION = u'Ameli website: French Health Insurance for Professionals'
    MAINTAINER = u'Christophe Lampin'
    EMAIL = 'weboob@lampin.net'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'
    BROWSER = AmeliProBrowser
    CONFIG = BackendConfig(ValueBackendPassword('login',
                                                label='numero de SS',
                                                masked=False),
                           ValueBackendPassword('password',
                                                label='Password',
                                                masked=True)
                           )

    def create_default_browser(self):
        self.logger.settings['save_responses'] = False # Set to True to help debugging
        return self.create_browser(self.config['login'].get(),
                                   self.config['password'].get())

    def iter_subscription(self):
        return self.browser.get_subscription_list()

    def get_subscription(self, _id):
        if not _id.isdigit():
            raise SubscriptionNotFound()
        subscription = self.browser.get_subscription(_id)
        if not subscription:
            raise SubscriptionNotFound()
        else:
            return subscription

    def iter_documents_history(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        return self.browser.iter_history(subscription)

    def get_details(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        return self.browser.get_details(subscription)

    def iter_documents(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        return self.browser.iter_documents()

    def get_document(self, id):
        bill = self.browser.get_document(id)
        if not bill:
            raise DocumentNotFound()
        else:
            return bill

    def download_document(self, bill):
        if not isinstance(bill, Bill):
            bill = self.get_document(bill)
        return self.browser.download_document(bill)



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Edouard Lambert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword

from .pages import LoginPage, MovementsPage, ProfilePage, PassExpiredPage


class LCLEnterpriseBrowser(LoginBrowser):
    BASEURL = 'https://entreprises.secure.lcl.fr'

    pass_expired = URL('/outil/IQEN/Authentication/forcerChangePassword', PassExpiredPage)
    login = URL('/outil/IQEN/Authentication/indexRedirect',
                '/outil/IQEN/Authentication/(?P<page>.*)', LoginPage)
    movements = URL('/outil/IQMT/mvt.Synthese/syntheseMouvementPerso',
                    '/outil/IQMT/mvt.Synthese', MovementsPage)
    profile = URL('/outil/IQGA/FicheUtilisateur/maFicheUtilisateur', ProfilePage)

    def __init__(self, *args, **kwargs):
        super(LCLEnterpriseBrowser, self).__init__(*args, **kwargs)
        self.accounts = None

    def deinit(self):
        if self.page.logged:
            self.login.go(page="logout")
            self.login.go(page="logoutOk")
            assert self.login.is_here(page="logoutOk") or self.login.is_here(page="sessionExpiree")
        super(LCLEnterpriseBrowser, self).deinit()

    def do_login(self):
        self.login.go().login(self.username, self.password)

        error = self.page.get_error() if self.login.is_here() else False

        if error:
            raise BrowserIncorrectPassword(error)

    @need_login
    def get_accounts_list(self):
        if not self.accounts:
            self.accounts = list(self.movements.go().iter_accounts())
        return self.accounts

    @need_login
    def get_history(self, account):
        if account._data:
            return self.open(account._url, data=account._data).page.iter_history()
        return self.movements.go().iter_history()

    @need_login
    def get_profile(self):
        return self.profile.go().get_profile()

    def get_cb_operations(self, account):
        raise NotImplementedError()

    def get_investment(self, account):
        raise NotImplementedError()


class LCLEspaceProBrowser(LCLEnterpriseBrowser):
    BASEURL = 'https://espacepro.secure.lcl.fr'



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from dateutil.parser import parse as _parse_dt

from weboob.tools.compat import urlsplit, parse_qs
from weboob.tools.date import local2utc


def url2id(url, nopost=False):
    v = urlsplit(url)
    pagename = v.path.split('/')[-1]
    args = parse_qs(v.query)
    if pagename == 'viewforum.php':
        return '%d' % int(args['f'][0])
    if pagename == 'viewtopic.php':
        if 'f' in args:
            s = '%d' % int(args['f'][0])
        else:
            s = '0'
        s += '.%d' % int(args['t'][0])
        if 'p' in args and not nopost:
            s += '.%d' % int(args['p'][0])
        return s

    return None


def id2url(id):
    v = str(id).split('.')
    if len(v) == 1:
        return 'viewforum.php?f=%d' % int(v[0])
    if len(v) == 2:
        return 'viewtopic.php?f=%d&t=%d' % (int(v[0]), int(v[1]))
    if len(v) == 3:
        return 'viewtopic.php?f=%d&t=%d&p=%d#p%d' % (int(v[0]),
                                                     int(v[1]),
                                                     int(v[2]),
                                                     int(v[2]))


def id2topic(id):
    try:
        return int(str(id).split('.')[1])
    except IndexError:
        return None


def rssid(id):
    return id


def parse_date(s):
    s = s.replace(u'Fév', 'Feb') \
         .replace(u'Avr', 'Apr') \
         .replace(u'Mai', 'May') \
         .replace(u'Juin', 'Jun') \
         .replace(u'Juil', 'Jul') \
         .replace(u'Aoû', 'Aug') \
         .replace(u'Ao\xfbt', 'Aug') \
         .replace(u'Déc', 'Dec')
    return local2utc(_parse_dt(s))



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Noé Rubinstein
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.tools.capabilities.gallery.genericcomicreader import GenericComicReaderModule, DisplayPage

__all__ = ['BatotoModule']


class BatotoModule(GenericComicReaderModule):
    NAME = 'batoto'
    DESCRIPTION = 'Batoto manga reading website'
    DOMAIN = 'www.batoto.com'
    BROWSER_PARAMS = dict(
        img_src_xpath="//img[@id='comic_page']/@src",
        page_list_xpath="(//select[@id='page_select'])[1]/option/@value")
    ID_REGEXP = r'[^/]+/[^/]+'
    URL_REGEXP = r'.+batoto.(?:com|net)/read/_/(%s).+' % ID_REGEXP
    ID_TO_URL = 'http://www.batoto.net/read/_/%s'
    PAGES = {URL_REGEXP: DisplayPage}



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Oleg Plakhotniuk
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.bank import AccountNotFound
from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword, BrowserUnavailable
from weboob.tools.compat import unquote

import ssl
import json
import os
from tempfile import mkstemp
from subprocess import STDOUT, CalledProcessError, check_output

from .pages import LoginProceedPage, LoginRedirectPage, \
                   SummaryPage, ActivityCashPage, ActivityCardPage, \
                   DocumentsPage, StatementPage, StatementsPage, \
                   StatementsEmbeddedPage, LoggedInPage, CodeRequestPage, \
                   CodeSubmitPage


__all__ = ['WellsFargo']


class WellsFargo(LoginBrowser):
    BASEURL = 'https://online.wellsfargo.com'
    TIMEOUT = 30
    MAX_RETRIES = 10
    login_proceed = URL('/das/cgi-bin/session.cgi\?screenid=SIGNON.*$',
                        '/login\?ERROR_CODE=.*LOB=CONS&$',
                        LoginProceedPage)
    login_redirect = URL('/das/cgi-bin/session.cgi\?screenid=SIGNON.*$',
                         '/login\?ERROR_CODE=.*LOB=CONS&$',
                         LoginRedirectPage)
    code_request = URL('https://oam.wellsfargo.com/oam/access'
                       '/twoFAAARDisplay\?OAM_TKN=.+$', CodeRequestPage)
    code_submit = URL('https://oam.wellsfargo.com/oam/access'
                      '/twoFAAARDisplay\?OAM_TKN=.+$',
                      'https://oam.wellsfargo.com/oam/access'
                      '/twoFAAARSubmitCode\?OAM_TKN=.+$', CodeSubmitPage)
    summary = URL('/das/channel/accountSummary$', SummaryPage)
    activity_cash = URL('/das/cgi-bin/session.cgi\?sessargs=.+$',
                        ActivityCashPage)
    activity_card = URL('/das/cgi-bin/session.cgi\?sessargs=.+$',
                        ActivityCardPage)
    documents = URL('https://connect.secure.wellsfargo.com'
                    '/accounts/start\?.+$', DocumentsPage)
    statements_embedded = URL('https://connect.secure.wellsfargo.com'
                    '/accounts/start\?.+$', StatementsEmbeddedPage)
    statements = URL('https://connect.secure.wellsfargo.com'
                     '/accounts/documents/statement/list.+$',
                     StatementsPage)
    statement = URL('https://connect.secure.wellsfargo.com'
                    '/accounts/documents/retrieve/.+$',
                    StatementPage)
    unknown = URL('/.*$', LoggedInPage) # E.g. random advertisement pages.

    def __init__(self, question1, answer1, question2, answer2,
                 question3, answer3, phone_last4, code_file, *args, **kwargs):
        super(WellsFargo, self).__init__(*args, **kwargs)
        self.question1 = question1
        self.answer1 = answer1
        self.question2 = question2
        self.answer2 = answer2
        self.question3 = question3
        self.answer3 = answer3
        self.phone_last4 = phone_last4
        self.code_file = code_file

    def do_login(self):
        '''
        There's a bunch of dynamically generated obfuscated JavaScript,
        which uses DOM. For now the easiest option seems to be to run it in
        PhantomJs.
        '''
        for i in xrange(self.MAX_RETRIES):
            scrf, scrn = mkstemp('.js')
            cookf, cookn = mkstemp('.json')
            os.write(scrf, LOGIN_JS % {
                'scriptTimeout': self.TIMEOUT*2,
                'resourceTimeout': self.TIMEOUT,
                'username': self.username,
                'password': self.password,
                'output': cookn,
                'question1': self.question1,
                'answer1': self.answer1,
                'question2': self.question2,
                'answer2': self.answer2,
                'question3': self.question3,
                'answer3': self.answer3})
            os.close(scrf)
            os.close(cookf)
            try:
                check_output(["phantomjs", scrn], stderr=STDOUT)
                with open(cookn) as cookf:
                    cookies = json.loads(cookf.read())
            except CalledProcessError:
                continue
            finally:
                os.remove(scrn)
                os.remove(cookn)
            self.session.cookies.clear()
            for c in cookies:
              for k in ['expiry', 'expires', 'httponly']:
                c.pop(k, None)
              c['value'] = unquote(c['value'])
              self.session.cookies.set(**c)
            self.summary.go()
            if self.page.logged:
                break
        else:
            raise BrowserIncorrectPassword

    def location(self, *args, **kwargs):
        """
        Wells Fargo inserts redirecting pages from time to time,
        so we should follow them whenever we see them.
        """
        r = super(WellsFargo, self).location(*args, **kwargs)
        if self.login_proceed.is_here():
            return self.page.proceed()
        elif self.login_redirect.is_here():
            return self.page.redirect()
        elif self.code_request.is_here():
            return self.page.request_code()
        elif self.code_submit.is_here():
            return self.page.submit_code()
        else:
            return r

    def prepare_request(self, req):
        """
        Wells Fargo uses TLS v1.0. See issue #1647 for details.
        """
        preq = super(WellsFargo, self).prepare_request(req)
        conn = self.session.adapters['https://'].get_connection(preq.url)
        conn.ssl_version = ssl.PROTOCOL_TLSv1
        return preq

    def get_account(self, id_):
        self.to_activity()
        if id_ not in self.page.accounts_ids():
            raise AccountNotFound()
        else:
            self.to_activity(id_)
            return self.page.get_account()

    def iter_accounts(self):
        self.to_activity()
        for id_ in self.page.accounts_ids():
            self.to_activity(id_)
            yield self.page.get_account()

    @need_login
    def to_summary(self):
        self.summary.stay_or_go()
        assert self.summary.is_here()

    def is_activity(self):
        return self.activity_cash.is_here() or self.activity_card.is_here()

    @need_login
    def to_activity(self, id_=None):
        if not self.is_activity():
            self.to_summary()
            self.page.to_activity()
            assert self.is_activity()
        if id_ and self.page.account_id() != id_:
            self.page.to_account(id_)
            assert self.is_activity()
            assert self.page.account_id() == id_

    @need_login
    def to_statements(self, id_=None, year=None):
        if not self.statements.is_here() \
        and not self.statements_embedded.is_here():
            self.to_summary()
            self.page.to_documents()
            if self.documents.is_here():
                self.page.to_statements()
                assert self.statements.is_here()
            else:
                assert self.statements_embedded.is_here()
        if id_ and self.page.parser().account_id() != id_:
            self.page.parser().to_account(id_)
            assert self.statements.is_here()
            assert self.page.parser().account_id() == id_
        if year and self.page.parser().year() != year:
            self.page.parser().to_year(year)
            assert self.statements.is_here()
            assert self.page.parser().year() == year

    @need_login
    def to_statement(self, uri):
        for i in xrange(self.MAX_RETRIES):
            self.location(uri)
            if self.statement.is_here():
                break
        else:
            raise BrowserUnavailable()

    def iter_history(self, account):
        self.to_activity(account.id)
        # Skip transactions on web page if we cannot apply
        # "since last statement" filter.
        # This might be the case, for example, if Wells Fargo
        # is processing the current statement:
        # "Since your credit card account statement is being processed,
        #  transactions grouped by statement period will not be available
        #  for up to seven days."
        # (www.wellsfargo.com, 2014-07-20)
        if self.page.since_last_statement():
            assert self.page.account_id() == account.id
            while True:
                for trans in self.page.iter_transactions():
                    yield trans
                if not self.page.next_():
                    break

        self.to_statements(account.id)
        for year in self.page.parser().years():
            self.to_statements(account.id, year)
            for stmt in self.page.parser().statements():
                self.to_statement(stmt)
                for trans in self.page.iter_transactions():
                    yield trans

LOGIN_JS = u'''\
var page = require('webpage').create();

page.settings.resourceTimeout = %(resourceTimeout)s*1000;
page.open('https://www.wellsfargo.com/');

var waitForForm = function() {
  var hasForm = page.evaluate(function(){
    return !!document.getElementById('frmSignon')
  });
  if (hasForm) {
    page.evaluate(function(){
      document.getElementById('userid').value = '%(username)s';
      document.getElementById('password').value = '%(password)s';
      document.getElementById('frmSignon').submit();
    });
  } else {
    setTimeout(waitForForm, 1000);
  }
}

var waitForQuestions = function() {
  var isQuestion = page.content.indexOf('Confirm Your Identity') != -1;
  if (isQuestion) {
    var questions = {
      "%(question1)s": "%(answer1)s",
      "%(question2)s": "%(answer2)s",
      "%(question3)s": "%(answer3)s"
    };
    for (var question in questions) {
      if (page.content.indexOf(question)) {
        page.evaluate(function(answer){
          document.getElementById('answer').value = answer;
          document.getElementById('command').submit.click();
        }, questions[question]);
      }
    }
  }
  setTimeout(waitForQuestions, 2000);
}

var waitForLogin = function() {
  var isSplash = page.content.indexOf('Splash Page') != -1;
  var hasSignOff = page.content.indexOf('Sign Off') != -1;
  if (isSplash || hasSignOff) {
    var cookies = JSON.stringify(phantom.cookies);
    require('fs').write('%(output)s', cookies, 'w');
    phantom.exit();
  } else {
    setTimeout(waitForLogin, 2000);
  }
}

waitForForm();
waitForQuestions();
waitForLogin();
setTimeout(function(){phantom.exit(-1);}, %(scriptTimeout)s*1000);
'''



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import PagesBrowser, URL

from .pages import BasePage
from weboob.tools.compat import urlparse

import re


class AgendaculturelBrowser(PagesBrowser):
    BASEURL = ''

    base = URL('https://www.agendaculturel.fr/search_bw',
               'https://(?P<region>\d{2}).agendaculturel.fr/(?P<_id>.*).html',
               'https://\d{2}.agendaculturel.fr/.*',
               BasePage)

    def __init__(self, place, *args, **kwargs):
        self.default_place = place
        PagesBrowser.__init__(self, *args, **kwargs)

    def set_base_url(self, place):
        if not place:
            place = self.default_place
        self.base.go(data={'query': place})
        parsed_uri = urlparse(self.page.url)
        self.BASEURL = '{uri.scheme}://{uri.netloc}/'.format(uri=parsed_uri)

    def list_events(self, place, date_from, date_to, categories=None):
        self.set_base_url(place)
        query_date_from = date_from.strftime('%Y%m')
        self.page.search_events(query_date_from)
        region = re.match('https://(\d{2}).agendaculturel.fr/.*', self.page.url).group(1)
        return self.page.list_events(region=region, date_from=date_from, date_to=date_to, categories=categories)

    def get_event(self, id, event=None):
        splitted_id = id.split('.')
        region = splitted_id[0]
        _id = splitted_id[1]
        return self.base.go(_id=_id, region=region).get_event(obj=event)



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value
from weboob.capabilities.calendar import CapCalendarEvent, CATEGORIES

from .browser import AgendaculturelBrowser
from .calendar import AgendaculturelEvent

__all__ = ['AgendaculturelModule']


class AgendaculturelModule(Module, CapCalendarEvent):
    NAME = 'agendaculturel'
    DESCRIPTION = u'agendaculturel website'
    MAINTAINER = u'Bezleputh'
    EMAIL = 'carton_ben@yahoo.fr'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'
    ASSOCIATED_CATEGORIES = [CATEGORIES.CONCERT, CATEGORIES.THEATRE, CATEGORIES.EXPO, CATEGORIES.AUTRE, CATEGORIES.FEST]
    BROWSER = AgendaculturelBrowser

    CONFIG = BackendConfig(Value('place', label='Default place'))

    def create_default_browser(self):
        return self.create_browser(self.config['place'].get())

    def get_event(self, _id):
        return self.browser.get_event(_id)

    def list_events(self, date_from, date_to=None):
        default_place = self.config['place'].get()
        return self.browser.list_events(default_place, date_from, date_to)

    def search_events(self, query):
        if self.has_matching_categories(query):
            return self.browser.list_events(query.city,
                                            query.start_date,
                                            query.end_date,
                                            query.categories)

    def fill_obj(self, event, fields):
        return self.browser.get_event(event.id, event)

    OBJECTS = {AgendaculturelEvent: fill_obj}



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Oleg Plakhotniuk
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.capabilities.bank.transactions import \
    AmericanTransaction as AmTr
from weboob.browser import LoginBrowser, URL, need_login
from weboob.browser.pages import HTMLPage
from weboob.capabilities.base import Currency
from weboob.capabilities.shop import Order, Item, Payment
from weboob.exceptions import BrowserIncorrectPassword

from datetime import datetime
from decimal import Decimal
from itertools import chain
import re


__all__ = ['VicSec']


class VicSecPage(HTMLPage):
    @property
    def logged(self):
        return bool(self.doc.xpath(
            '//a[@href="https://www.victoriassecret.com/account/profile"]'))


class LoginPage(VicSecPage):
    def login(self, email, password):
        form = self.get_form(name='accountLogonForm')
        form['j_username'] = email
        form['j_password'] = password
        form.submit()


class HistoryPage(VicSecPage):
    def iter_orders(self):
        return [onum for date, onum in sorted(
                chain(self.orders(), self.returns()), reverse=True)]

    def orders(self):
        for tr in self.doc.xpath('//table[@class="order-status"]/tbody[1]/tr'):
            date = datetime.strptime(tr.xpath('td[1]/text()')[0], '%m/%d/%Y')
            num = tr.xpath('td[2]/a/text()')[0]
            status = tr.xpath('td[4]/span/text()')[0]
            if status == u'Delivered':
                yield date, num

    def returns(self):
        for tr in self.doc.xpath('//table[@class="order-status"]/tbody[3]/tr'):
            num = tr.xpath('td[1]/a/text()')[0]
            date = datetime.strptime(tr.xpath('td[2]/text()')[0], '%m/%d/%Y')
            status = tr.xpath('td[4]/span/text()')[0]
            if status == u'Complete':
                yield date, num


class OrderPage(VicSecPage):
    def order(self):
        order = Order(id=self.order_number())
        order.date = self.order_date()
        order.tax = self.tax()
        order.discount = self.discount()
        order.shipping = self.shipping()
        return order

    def payments(self):
        for tr in self.doc.xpath('//tbody[@class="payment-summary"]'
                        '//th[text()="Payment Summary"]/../../../tbody/tr'):
            method = tr.xpath('td[1]/text()')[0]
            amnode = tr.xpath('td[2]')[0]
            amsign = -1 if amnode.xpath('em') else 1
            amount = amnode.text_content().strip()
            pmt = Payment()
            pmt.date = self.order_date()
            pmt.method = unicode(method)
            pmt.amount = amsign * AmTr.decimal_amount(amount)
            if pmt.method not in [u'Funds to be applied on backorder']:
                yield pmt

    def items(self):
        for tr in self.doc.xpath('//tbody[@class="order-items"]/tr'):
            label = tr.xpath('*//h1')[0].text_content().strip()
            price = AmTr.decimal_amount(re.match(r'^\s*([^\s]+)(\s+.*)?',
                tr.xpath('*//div[@class="price"]')[0].text_content(),
                re.DOTALL).group(1))
            url = 'http:' + tr.xpath('*//img/@src')[0]
            item = Item()
            item.label = unicode(label)
            item.url = unicode(url)
            item.price = price
            yield item

    def is_void(self):
        return not self.doc.xpath('//tbody[@class="order-items"]/tr')

    def order_number(self):
        return self.order_info(u'Order Number')

    def order_date(self):
        return datetime.strptime(self.order_info(u'Order Date'), '%m/%d/%Y')

    def tax(self):
        return self.payment_part(u'Sales Tax')

    def shipping(self):
        return self.payment_part(u'Shipping & Handling')

    def order_info(self, which):
        info = self.doc.xpath('//p[@class="orderinfo"]'
                             )[0].text_content()
        return re.match(u'.*%s:\\s+([^\\s]+)\\s'%which,info,re.DOTALL).group(1)

    def discount(self):
        # Sometimes subtotal doesn't add up with items.
        # I saw that "Special Offer" was actually added to the item's price,
        # instead of being subtracted. Looks like a bug on VS' side.
        # To compensate for it I'm correcting discount value.
        dcnt = self.payment_part(u'Special Offer')
        subt = self.payment_part(u'Merchandise Subtotal')
        rett = self.payment_part(u'Return Merchandise Total')
        items = sum(i.price for i in self.items())
        return dcnt + subt + rett - items

    def payment_part(self, which):
        # The numbers notation on VS is super wierd.
        # Sometimes negative amounts are represented by <em> element.
        for node in self.doc.xpath('//tbody[@class="payment-summary"]'
                              '//td[contains(text(),"%s")]/../td[2]' % which):
            strv = node.text_content().strip()
            v = Decimal(0) if strv == u'FREE' else AmTr.decimal_amount(strv)
            return -v if node.xpath('em') and v > 0 else v
        return Decimal(0)


class VicSec(LoginBrowser):
    BASEURL = 'https://www.victoriassecret.com'
    login = URL(r'/account/signin/overlay$', LoginPage)
    history = URL(r'/account/orderlist$', HistoryPage)
    order = URL(r'/account/orderstatus/submit/view\?emailId=(?P<email>.*)&orderNumber=(?P<order_num>\d+)$',
                r'/account/orderstatus.*$',
                OrderPage)
    unknown = URL(r'/.*$', VicSecPage)

    def get_currency(self):
        # Victoria's Secret uses only U.S. dollars.
        return Currency.get_currency(u'$')

    def get_order(self, id_):
        return self.to_order(id_).order()

    def iter_orders(self):
        for order in self.to_history().iter_orders():
            if not self.to_order(order).is_void():
                yield self.page.order()

    def iter_payments(self, order):
        return self.to_order(order.id).payments()

    def iter_items(self, order):
        return self.to_order(order.id).items()

    @need_login
    def to_history(self):
        self.history.stay_or_go()
        assert self.history.is_here()
        return self.page

    @need_login
    def to_order(self, order_num):
        self.order.stay_or_go(order_num=order_num, email=self.username.upper())
        assert self.order.is_here(order_num=order_num,
                                  email=self.username.upper())
        return self.page

    def do_login(self):
        self.session.cookies.clear()
        # Need to go there two times. Perhaps because of cookies...
        self.location('/secureoverlay/wrapper/medium/account/signin/overlay/show')
        self.login.go()
        self.login.go().login(self.username, self.password)
        self.history.go()
        if not self.history.is_here():
            raise BrowserIncorrectPassword()



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from .base import Capability, BaseObject, StringField, FloatField


__all__ = ['IpLocation', 'CapGeolocIp']


class IpLocation(BaseObject):
    """
    Represents the location of an IP address.
    """
    city =      StringField('City')
    region =    StringField('Region')
    zipcode =   StringField('Zip code')
    country =   StringField('Country')
    lt =        FloatField('Latitude')
    lg =        FloatField('Longitude')
    osmlink =   StringField('Link to OpenStreetMap location page')
    host =      StringField('Hostname')
    tld =       StringField('Top Level Domain')
    isp =       StringField('Internet Service Provider')


class CapGeolocIp(Capability):
    """
    Access information about IP addresses database.
    """

    def get_location(self, ipaddr):
        """
        Get location of an IP address.

        :param ipaddr: IP address
        :type ipaddr: str
        :rtype: :class:`IpLocation`
        """
        raise NotImplementedError()



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2015 Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword
from .pages import DocumentsPage, HomePage, LoginPage, ProfilePage, SendSMSPage, SendSMSErrorPage, UselessPage

from weboob.capabilities.messages import CantSendMessage

__all__ = ['BouyguesBrowser']


class BouyguesBrowser(LoginBrowser):
    BASEURL = 'https://www.mon-compte.bouyguestelecom.fr/'
    TIMEOUT = 20

    login = URL('cas/login', LoginPage)
    home = URL('https://www.bouyguestelecom.fr/mon-compte', HomePage)
    profile = URL('https://api-mc.bouyguestelecom.fr/client/me/header.json', ProfilePage)
    documents = URL('https://www.bouyguestelecom.fr/parcours/mes-factures',
                    'https://www.bouyguestelecom.fr/parcours/mes-factures/historique\?no_reference=(?P<ref>)', DocumentsPage)

    sms_page = URL('http://www.mobile.service.bbox.bouyguestelecom.fr/services/SMSIHD/sendSMS.phtml',
                   'http://www.mobile.service.bbox.bouyguestelecom.fr/services/SMSIHD/confirmSendSMS.phtml',
                   SendSMSPage)
    confirm = URL('http://www.mobile.service.bbox.bouyguestelecom.fr/services/SMSIHD/resultSendSMS.phtml', UselessPage)
    sms_error_page = URL('http://www.mobile.service.bbox.bouyguestelecom.fr/services/SMSIHD/SMS_erreur.phtml',
                         SendSMSErrorPage)

    def do_login(self):
        self.login.go()

        if self.home.is_here():
            return

        self.page.login(self.username, self.password)

        if not self.home.is_here():
            raise BrowserIncorrectPassword()

    @need_login
    def post_message(self, message):
        self.sms_page.go()

        if self.sms_error_page.is_here():
            raise CantSendMessage(self.page.get_error_message())

        receivers = ";".join(list(message.receivers)) if message.receivers else self.username
        self.page.send_sms(message, receivers)

        if self.sms_error_page.is_here():
            raise CantSendMessage(self.page.get_error_message())

        self.confirm.open()

    @need_login
    def get_subscription_list(self):
        return self.profile.stay_or_go().get_list()

    @need_login
    def iter_documents(self, subscription):
        ref = self.documents.go().get_ref(subscription.label)

        if not ref:
            return iter([])

        return self.documents.go(ref=ref).get_documents(subid=subscription.id)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright(C) 2013      Laurent Bachelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

import argparse
import subprocess
import datetime
import importlib
import os
import sys
import codecs
from mako.lookup import TemplateLookup

MODULE_PATH = os.getenv(
    'MODULE_PATH',
    os.path.realpath(os.path.join(os.path.dirname(__file__), '../modules')))
TEMPLATE_PATH = os.getenv(
    'TEMPLATE_PATH',
    os.path.realpath(os.path.join(os.path.dirname(__file__), 'boilerplate_data')))
VERSION = '1.3'

TEMPLATES = TemplateLookup(directories=[TEMPLATE_PATH])


def u8(s):
    if isinstance(s, bytes):
        return s.decode('utf-8')
    return s


def gitconfig(entry):
    return u8(subprocess.check_output('git config -z --get %s' % entry, shell=True)[:-1])


def write(target, contents):
    if not os.path.isdir(os.path.dirname(target)):
        os.makedirs(os.path.dirname(target))
    if os.path.exists(target):
        print("%s already exists." % target, file=sys.stderr)
        sys.exit(4)
    with codecs.open(target, mode='w', encoding='utf-8') as f:
        f.write(contents)
    print('Created %s' % target)


class Recipe(object):
    @classmethod
    def configure_subparser(cls, subparsers):
        subparser = subparsers.add_parser(cls.NAME)
        subparser.add_argument('name', help='Module name')
        subparser.set_defaults(recipe=cls)
        return subparser

    def __init__(self, args):
        self.name = args.name.lower().replace(' ', '')
        self.classname = args.name.title().replace(' ', '')
        self.year = datetime.date.today().year
        self.author = args.author
        self.email = args.email
        self.version = VERSION

    def write(self, filename, contents):
        return write(os.path.join(MODULE_PATH, self.name, filename), contents)

    def template(self, name, **kwargs):
        if '.' not in name:
            name += '.py'
        return TEMPLATES.get_template(name) \
            .render(r=self,
                    # workaround, as it's also a mako directive
                    coding='# -*- coding: utf-8 -*-',
                    **kwargs)

    def generate(self):
        raise NotImplementedError()


class BaseRecipe(Recipe):
    NAME = 'base'

    def generate(self):
        self.write('__init__.py', self.template('init'))
        self.write('module.py', self.template('base_module'))
        self.write('browser.py', self.template('base_browser'))
        self.write('pages.py', self.template('base_pages'))
        self.write('test.py', self.template('base_test'))


class CapRecipe(Recipe):
    NAME = 'cap'

    LINES = {'def'    :  '    def %s%s:',
             'docbound': '        """',
             'docline':  '        %s',
             'body'   :  '        raise NotImplementedError()'
             }

    def __init__(self, args):
        super(CapRecipe, self).__init__(args)
        self.capname = args.capname

    @classmethod
    def configure_subparser(cls, subparsers):
        subparser = super(CapRecipe, cls).configure_subparser(subparsers)
        subparser.add_argument('capname', help='Capability name')
        return subparser

    def find_module_cap(self):
        if '.' not in self.capname:
            return self.search_cap()

        PREFIX = 'weboob.capabilities.'
        if not self.capname.startswith(PREFIX):
            self.capname = PREFIX + self.capname

        try:
            self.capmodulename, self.capname = self.capname.rsplit('.', 1)
        except ValueError:
            self.error('Cap name must be in format module.CapSomething or CapSomething')

        try:
            module = importlib.import_module(self.capmodulename)
        except ImportError:
            self.error('Module %r not found' % self.capmodulename)
        try:
            cap = getattr(module, self.capname)
        except AttributeError:
            self.error('Module %r has no such capability %r' % (self.capmodulename, self.capname))
        return cap

    def search_cap(self):
        import pkgutil
        import weboob.capabilities

        modules = pkgutil.walk_packages(weboob.capabilities.__path__, prefix='weboob.capabilities.')
        for _, capmodulename, __ in modules:
            module = importlib.import_module(capmodulename)
            if hasattr(module, self.capname):
                self.capmodulename = capmodulename
                return getattr(module, self.capname)

        self.error('Capability %r not found' % self.capname)

    def error(self, message):
        print(message, file=sys.stderr)
        sys.exit(1)

    def methods_code(self, klass):
        import inspect
        import re

        codes = []

        for name, member in inspect.getmembers(klass):
            if inspect.ismethod(member):
                argspec = inspect.getargspec(member)
                args = inspect.formatargspec(*argspec)

                code = []
                code.append(self.LINES['def'] % (name, args))
                doc = inspect.getdoc(member)
                if doc:
                    code.append(self.LINES['docbound'])
                    for line in doc.split('\n'):
                        if line:
                            line = re.sub('"""', '\\"\\"\\"', line)
                            code.append(self.LINES['docline'] % line)
                        else:
                            code.append('')
                    code.append(self.LINES['docbound'])
                code.append(self.LINES['body'])
                codes.append('\n'.join(code))

        return '\n\n'.join(codes)

    def generate(self):
        cap = self.find_module_cap()

        self.methods_code = self.methods_code(cap)

        self.write('__init__.py', self.template('init'))
        self.write('module.py', self.template('cap_module'))
        self.write('browser.py', self.template('base_browser'))
        self.write('pages.py', self.template('base_pages'))
        self.write('test.py', self.template('base_test'))


class ComicRecipe(Recipe):
    NAME = 'comic'

    def generate(self):
        self.write('__init__.py', self.template('init'))
        self.write('module.py', self.template('comic_module'))


class ComicTestRecipe(Recipe):
    NAME = 'comic.test'

    @classmethod
    def configure_subparser(cls, subparsers):
        subparser = super(ComicTestRecipe, cls).configure_subparser(subparsers)
        subparser.add_argument('download_id', help='Download ID')
        return subparser

    def __init__(self, args):
        super(ComicTestRecipe, self).__init__(args)
        self.download_id = args.download_id

    def generate(self):
        self.write('test.py', self.template('comic_test'))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-a', '--author',
        default=gitconfig('user.name'), type=u8)
    parser.add_argument(
        '-e', '--email',
        default=gitconfig('user.email'), type=u8)
    subparsers = parser.add_subparsers()

    recipes = [BaseRecipe, ComicRecipe, ComicTestRecipe, CapRecipe]
    for recipe in recipes:
        recipe.configure_subparser(subparsers)

    args = parser.parse_args()

    recipe = args.recipe(args)
    recipe.generate()

if __name__ == '__main__':
    main()



<%inherit file="layout.py"/>
from weboob.browser import PagesBrowser, URL

from .pages import Page1, Page2


class ${r.classname}Browser(PagesBrowser):
    BASEURL = 'http://www.${r.name}.com'

    page1 = URL('/page1\?id=(?P<id>.+)', Page1)
    page2 = URL('/page2', Page2)

    def get_stuff(self, _id):
        self.page1.go(id=_id)

        assert self.page1.is_here()
        self.page.do_stuff(_id)

        assert self.page2.is_here()
        return self.page.do_more_stuff()



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.lyrics import SongLyrics
from weboob.capabilities.base import NotLoaded, NotAvailable

from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.pages import HTMLPage
from weboob.browser.filters.standard import Regexp, CleanText
from weboob.browser.filters.html import CleanHTML


class HomePage(HTMLPage):
    def search_lyrics(self, pattern):
        form = self.get_form(xpath='//form[@id="search-form-round"]')
        form['search'] = pattern
        form.submit()


class ResultsPage(HTMLPage):
    @method
    class iter_song_lyrics(ListElement):
        item_xpath = '//h2[text()="Chansons"]/following-sibling::div[position() <= 2]//tr'

        class item(ItemElement):
            klass = SongLyrics

            def obj_id(self):
                href = CleanText('.//td[has-class("song-name")]//a/@href')(self)
                aid = href.split('/')[-2]
                sid = href.split('/')[-1].replace('paroles-','')
                id = '%s|%s'%(aid, sid)
                return id

            obj_title = CleanText('.//td[has-class("song-name")]',
                    default=NotAvailable)
            obj_artist = CleanText('.//td[has-class("song-artist")]',
                    default=NotAvailable)
            obj_content = NotLoaded

    def get_artist_ids(self):
        artists_href = self.doc.xpath('//h2[text()="Artiste"]/following-sibling::div[position() <= 2]//tr//a/@href')
        aids = [href.split('/')[-1] for href in artists_href]
        return aids


class ArtistSongsPage(HTMLPage):
    @method
    class iter_lyrics(ListElement):
        item_xpath = '//div[@id="main"]//div[has-class("song-listing-extra")]//td[has-class("song-name")]//a'

        class item(ItemElement):
            klass = SongLyrics

            obj_title = CleanText('.',
                    default=NotAvailable)
            obj_artist = Regexp(CleanText('//div[has-class("breadcrumb")]//span[has-class("breadcrumb-current")]'),
                    'Paroles (.*)')
            obj_content = NotLoaded
            def obj_id(self):
                href = CleanText('./@href')(self)
                aid = href.split('/')[-2]
                sid = href.split('/')[-1].replace('paroles-','')
                id = '%s|%s'%(aid, sid)
                return id


class SongLyricsPage(HTMLPage):
    @method
    class get_lyrics(ItemElement):
        klass = SongLyrics

        def obj_id(self):
            subid = self.page.url.replace('paroles-','').split('/')[-2:]
            id = '%s|%s'%(subid[0], subid[1])
            return id
        obj_content = CleanText(CleanHTML('//div[has-class("song-text")]', default=NotAvailable), newlines=False)
        obj_title = CleanText('//span[@property="v:name"]', default=NotAvailable)
        obj_artist = CleanText('//span[@property="v:artist"]', default=NotAvailable)




# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import string

from weboob.capabilities.torrent import Torrent
from weboob.capabilities.base import NotAvailable, NotLoaded
from weboob.deprecated.browser import Page
from weboob.tools.misc import get_bytes_size


class TorrentsPage(Page):
    def iter_torrents(self):
        for div in self.parser.select(self.document.getroot(),'div.list_tor'):
            name = NotAvailable
            size = NotAvailable
            seeders = NotAvailable
            leechers = NotAvailable
            right_div = self.parser.select(div,'div.list_tor_right',1)
            try:
                seeders = int(self.parser.select(right_div,'b.green',1).text)
            except ValueError:
                seeders = 0
            try:
                leechers = int(self.parser.select(right_div,'b.red',1).text)
            except ValueError:
                leechers = 0
            sizep = self.parser.select(right_div,'p')[0]
            sizespan = self.parser.select(sizep,'span')[0]
            nsize = float(sizespan.text_content().split(':')[1].split()[0])
            usize = sizespan.text_content().split()[-1].upper()
            size = get_bytes_size(nsize,usize)
            a = self.parser.select(div,'a.list_tor_title',1)
            href = a.attrib.get('href','')
            name = unicode(a.text_content())
            id = unicode(href.strip('/').split('.html')[0])
            torrent = Torrent(id,name)
            torrent.url = NotLoaded
            torrent.filename = id
            torrent.magnet = NotLoaded
            torrent.size = size
            torrent.seeders = seeders
            torrent.leechers = leechers
            torrent.description = NotLoaded
            torrent.files = NotLoaded
            yield torrent


class TorrentPage(Page):
    def get_torrent(self):
        seed = 0
        leech = 0
        description = NotAvailable
        url = NotAvailable
        magnet = NotAvailable
        title = NotAvailable
        id = unicode(self.browser.geturl().split('.html')[0].split('/')[-1])

        div = self.parser.select(self.document.getroot(),'div#middle_content',1)
        title = u'%s'%self.parser.select(self.document.getroot(),'div#middle_content > h1',1).text
        slblock_values = self.parser.select(div,'div.sl_block b')
        if len(slblock_values) >= 2:
            seed = slblock_values[0].text
            leech = slblock_values[1].text
        href_t = self.parser.select(div,'a.down',1).attrib.get('href','')
        url = u'http://%s%s'%(self.browser.DOMAIN,href_t)
        magnet = unicode(self.parser.select(div,'a.magnet',1).attrib.get('href',''))

        divtabs = self.parser.select(div,'div#tabs',1)
        files_div = self.parser.select(divtabs,'div.body > div.doubleblock > div.leftblock')
        files = []
        if len(files_div) > 0:
            size_text = self.parser.select(files_div,'h5',1).text
            for b in self.parser.select(files_div,'b'):
                div = b.getparent()
                files.append(div.text_content())
        else:
            size_text = self.parser.select(divtabs,'h5',1).text_content()
        size_text = size_text.split('(')[1].split(')')[0].strip()
        size = float(size_text.split(',')[1].strip(string.letters))
        u = size_text.split(',')[1].strip().translate(None,string.digits).strip('.').strip().upper()
        div_desc = self.parser.select(divtabs,'div#descriptionContent')
        if len(div_desc) > 0:
            description = unicode(div_desc[0].text_content())

        torrent = Torrent(id, title)
        torrent.url = url
        torrent.filename = id
        torrent.magnet = magnet
        torrent.size = get_bytes_size(size, u)
        torrent.seeders = int(seed)
        torrent.leechers = int(leech)
        torrent.description = description
        torrent.files = files
        return torrent



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Laurent Bachelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import csv

from weboob.tools.log import getLogger
from weboob.tools.compat import basestring, unicode
from .iparser import IParser


class Csv(object):
    """
    CSV parser result.
    header contains the first row if it is a header
    rows contains the raw rows
    drows contains the rows with cells indexed by header title
    """

    def __init__(self):
        self.header = None
        self.rows = []
        self.drows = []


class CsvParser(IParser):
    """
    CSV Parser.
    Since CSV files are not normalized, this parser is intended to be derived.
    """
    DIALECT = 'excel'
    FMTPARAMS = {}

    """
    If True, will consider the first line as a header.
    This means the rows will be also available as dictionnaries.
    """
    HEADER = False

    def parse(self, data, encoding=None):
        reader = csv.reader(data, dialect=self.DIALECT, **self.FMTPARAMS)
        c = Csv()
        try:
            for row in reader:
                row = self.decode_row(row, encoding)
                if c.header is None and self.HEADER:
                    c.header = row
                else:
                    c.rows.append(row)
                    if c.header:
                        drow = {}
                        for i, cell in enumerate(row):
                            drow[c.header[i]] = cell
                        c.drows.append(drow)
        except csv.Error as error:
            # If there are errors in CSV, for example the file is truncated, do
            # not crash as there already are lines parsed.
            logger = getLogger('csv')
            logger.warning('Error during parse of CSV: %s', error)
        return c

    def decode_row(self, row, encoding):
        if encoding:
            return [unicode(cell, encoding) for cell in row]
        else:
            return row

    def tostring(self, element):
        if not isinstance(element, basestring):
            return unicode(element)
        return element



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Nicolas Duhamel
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re

from weboob.capabilities.video import CapVideo, BaseVideo
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value

from .browser import CanalplusBrowser
from .video import CanalplusVideo

from weboob.capabilities.collection import CapCollection


__all__ = ['CanalplusModule']


class CanalplusModule(Module, CapVideo, CapCollection):
    NAME = 'canalplus'
    MAINTAINER = u'Nicolas Duhamel'
    EMAIL = 'nicolas@jombi.fr'
    VERSION = '1.3'
    DESCRIPTION = 'Canal Plus French TV'
    LICENSE = 'AGPLv3+'
    CONFIG = BackendConfig(Value('quality', label='Quality of videos', choices=['hd', 'sd'], default='hd'))
    BROWSER = CanalplusBrowser

    def create_default_browser(self):
        return self.create_browser(quality=self.config['quality'].get())

    def search_videos(self, pattern, sortby=CapVideo.SEARCH_RELEVANCE, nsfw=False):
        return self.browser.search_videos(pattern)

    def get_video(self, _id):
        m = re.match('https?://www\.canal-?plus\.fr/.*\?vid=(\d+)', _id)
        if m:
            _id = m.group(1)
        return self.browser.get_video(_id)

    def fill_video(self, video, fields):
        if fields != ['thumbnail']:
            # if we don't want only the thumbnail, we probably want also every fields
            video = self.browser.get_video(CanalplusVideo.id2url(video.id), video)
        if 'thumbnail' in fields and video.thumbnail:
            video.thumbnail.data = self.browser.open(video.thumbnail.url).content
        return video

    OBJECTS = {CanalplusVideo: fill_video}

    def iter_resources(self, objs, split_path):
        if BaseVideo in objs:
            return self.browser.iter_resources(split_path)



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Vincent Paredes
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.pages import HTMLPage, LoggedPage
from weboob.browser.filters.standard import CleanDecimal, CleanText, Env, Format
from weboob.browser.elements import ListElement, ItemElement, method
from weboob.browser.filters.html import Attr
from weboob.capabilities.bill import Bill, Subscription
from weboob.tools.date import parse_french_date

class HomePage(LoggedPage, HTMLPage):
    @method
    class get_list(ListElement):
        item_xpath = '//div[@id="divAccueilInformationClient"]//div[@id="divInformationClient"]'
        class item(ItemElement):
            klass = Subscription

            obj_subscriber = CleanText('.//div[@id="divlblTitleFirstNameLastName"]/span')
            obj_id = CleanText('.//span[2]')
            obj_label = CleanText('.//div[@id="divlblTitleFirstNameLastName"]/span')


class LoginPage(HTMLPage):
    def login(self, username, password):
        form = self.get_form(xpath='//form[@id="aspnetForm"]')
        form["ctl00$ctl00$cphMainContent$cphMainContent$txbMail"] = username
        form["ctl00$ctl00$cphMainContent$cphMainContent$txbPassword"] = password
        form["__EVENTTARGET"] = "ctl00$ctl00$cphMainContent$cphMainContent$butConnexion"
        form["ctl00_ctl00_actScriptManager_HiddenField"] = ";;LIBLDLC:fr-FR:2a6faad4-8912-415f-8586-e1088a6418ed:e5e73cb5;AjaxControlToolkit, Version=3.5.40412.0, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e:fr-FR:f0f17dea-2bbf-45d7-9f07-8361cd7f1424:5546a2b:475a4ef5:d2e10b12;LIBLDLC, Version=1.1.3.0, Culture=neutral, PublicKeyToken=null:fr-FR:2a6faad4-8912-415f-8586-e1088a6418ed::::70413c6f;AjaxControlToolkit, Version=3.5.40412.0, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e:fr-FR:f0f17dea-2bbf-45d7-9f07-8361cd7f1424:effe2a26:751cdd15:dfad98a5:1d3ed089:497ef277:a43b07eb:3cf12cf1"
        form.submit()


class BillsPage(LoggedPage, HTMLPage):
    def get_range(self):
        for value in self.doc.xpath('//div[@class="commandListing content clearfix"]//select/option/@value'):
            yield value

    @method
    class get_documents(ListElement):
        item_xpath = '//table[@id="TopListing"]//tr'

        class item(ItemElement):
            klass = Bill

            obj_id = Format('%s_%s', Env('subid'), CleanText('./td[3]'))
            obj_url = Attr('./td[@class="center" or @class="center pdf"]/a', 'href')
            obj_date = Env('date')
            obj_format = u"pdf"
            obj_type = u"bill"
            obj_price = CleanDecimal('./td[@class="center montant"]/span', replace_dots=True)

            def parse(self, el):
                self.env['date'] = parse_french_date(el.xpath('./td[2]')[0].text).date()

            def condition(self):
                return CleanText().filter(self.el.xpath('.//td')[-1]) != "" and len(self.el.xpath('./td[@class="center" or @class="center pdf"]/a/@href')) == 1




# -*- coding: utf-8 -*-

# Copyright(C) 2013 Alexandre Lissy
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import datetime

from weboob.capabilities.travel import RoadmapFilters
from weboob.tools.test import BackendTest


class JVMalinTest(BackendTest):
    MODULE = 'jvmalin'

    def test_roadmap_cities(self):
        filters = RoadmapFilters()
        roadmap = list(self.backend.iter_roadmap('Tours', 'Orléans', filters))
        self.assertTrue(len(roadmap) > 0)

    def test_roadmap_stop_intercity(self):
        filters = RoadmapFilters()
        roadmap = list(self.backend.iter_roadmap('Tours Jean-Jaurès', 'Orléans', filters))
        self.assertTrue(len(roadmap) > 0)

    def test_roadmap_stop_intracity(self):
        filters = RoadmapFilters()
        roadmap = list(self.backend.iter_roadmap('Tours Jean-Jaurès', 'Polytech Tours', filters))
        self.assertTrue(len(roadmap) > 0)

    def test_roadmap_stop_intracity2(self):
        filters = RoadmapFilters()
        roadmap = list(self.backend.iter_roadmap('J.P.Rameau', 'Polytech Tours', filters))
        self.assertTrue(len(roadmap) > 0)

    def test_roadmap_names(self):
        filters = RoadmapFilters()
        roadmap = list(self.backend.iter_roadmap('Artannes Mairie', 'Château de Blois', filters))
        self.assertTrue(len(roadmap) > 0)

    def test_roadmap_long(self):
        filters = RoadmapFilters()
        roadmap = list(self.backend.iter_roadmap('Chartres', 'Ballan-Miré', filters))
        self.assertTrue(len(roadmap) > 0)

    def test_roadmap_departure(self):
        filters = RoadmapFilters()
        filters.departure_time = datetime.datetime.now() + datetime.timedelta(days=1)
        roadmap = list(self.backend.iter_roadmap('Chartres', 'Ballan-Miré', filters))
        self.assertTrue(len(roadmap) > 0)

    def test_roadmap_arrival(self):
        filters = RoadmapFilters()
        filters.arrival_time = datetime.datetime.now() + datetime.timedelta(days=1)
        roadmap = list(self.backend.iter_roadmap('Chartres', 'Ballan-Miré', filters))
        self.assertTrue(len(roadmap) > 0)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from PyQt5.QtGui import QImage, QIcon, QPixmap
from PyQt5.QtWidgets import QWidget, QTreeWidgetItem
from PyQt5.QtCore import Qt, pyqtSignal as Signal, pyqtSlot as Slot

from weboob.capabilities.base import NotLoaded
from weboob.tools.compat import range
from weboob.tools.application.qt5 import QtDo, HTMLDelegate

from .ui.events_ui import Ui_Events


class EventsWidget(QWidget):
    display_contact = Signal(object)

    def __init__(self, weboob, parent=None):
        super(EventsWidget, self).__init__(parent)
        self.ui = Ui_Events()
        self.ui.setupUi(self)

        self.weboob = weboob
        self.photo_processes = {}
        self.event_filter = None

        self.ui.eventsList.itemDoubleClicked.connect(self.eventDoubleClicked)
        self.ui.typeBox.currentIndexChanged.connect(self.typeChanged)
        self.ui.refreshButton.clicked.connect(self.refreshEventsList)

        self.ui.eventsList.setItemDelegate(HTMLDelegate())
        self.ui.eventsList.sortByColumn(1, Qt.DescendingOrder)

    def load(self):
        self.refreshEventsList()

    @Slot(int)
    def typeChanged(self, i):
        if self.ui.refreshButton.isEnabled():
            self.refreshEventsList()

    @Slot()
    def refreshEventsList(self):
        self.ui.eventsList.clear()
        self.ui.refreshButton.setEnabled(False)
        if self.ui.typeBox.currentIndex() >= 0:
            # XXX strangely, in gotEvent() in the loop to check if there is already the
            # event type to try to introduce it in list, itemData() returns the right value.
            # But, I don't know why, here, it will ALWAYS return None...
            # So the filter does not work currently.
            self.events_filter = self.ui.typeBox.itemData(self.ui.typeBox.currentIndex())
        else:
            self.event_filter = None
        self.ui.typeBox.setEnabled(False)
        self.ui.typeBox.clear()
        self.ui.typeBox.addItem('All', None)

        def finished():
            self.ui.refreshButton.setEnabled(True)
            self.ui.typeBox.setEnabled(True)

        self.process = QtDo(self.weboob, self.gotEvent, fb=finished)
        self.process.do('iter_events')

    def setPhoto(self, contact, item):
        if not contact:
            return False

        try:
            self.photo_processes.pop(contact.id, None)
        except KeyError:
            pass

        img = None
        for photo in contact.photos.values():
            if photo.thumbnail_data:
                img = QImage.fromData(photo.thumbnail_data)
                break

        if img:
            item.setIcon(0, QIcon(QPixmap.fromImage(img)))
            self.ui.eventsList.resizeColumnToContents(0)
            return True

        return False

    def gotEvent(self, event):
        found = False
        for i in range(self.ui.typeBox.count()):
            s = self.ui.typeBox.itemData(i)
            if s == event.type:
                found = True
        if not found:
            print(event.type)
            self.ui.typeBox.addItem(event.type.capitalize(), event.type)
            if event.type == self.event_filter:
                self.ui.typeBox.setCurrentIndex(self.ui.typeBox.count()-1)

        if self.event_filter and self.event_filter != event.type:
            return

        if not event.contact:
            return

        contact = event.contact
        contact.backend = event.backend
        status = ''

        if contact.status == contact.STATUS_ONLINE:
            status = u'Online'
            status_color = 0x00aa00
        elif contact.status == contact.STATUS_OFFLINE:
            status = u'Offline'
            status_color = 0xff0000
        elif contact.status == contact.STATUS_AWAY:
            status = u'Away'
            status_color = 0xffad16
        else:
            status = u'Unknown'
            status_color = 0xaaaaaa

        if contact.status_msg:
            status += u' — %s' % contact.status_msg

        name = '<h2>%s</h2><font color="#%06X">%s</font><br /><i>%s</i>' % (contact.name, status_color, status, event.backend)
        date = event.date.strftime('%Y-%m-%d %H:%M')
        type = event.type
        message = event.message

        item = QTreeWidgetItem(None, [name, date, type, message])
        item.setData(0, Qt.UserRole, event)
        if contact.photos is NotLoaded:
            process = QtDo(self.weboob, lambda c: self.setPhoto(c, item))
            process.do('fillobj', contact, ['photos'], backends=contact.backend)
            self.photo_processes[contact.id] = process
        elif len(contact.photos) > 0:
            if not self.setPhoto(contact, item):
                photo = list(contact.photos.values())[0]
                process = QtDo(self.weboob, lambda p: self.setPhoto(contact, item))
                process.do('fillobj', photo, ['thumbnail_data'], backends=contact.backend)
                self.photo_processes[contact.id] = process

        self.ui.eventsList.addTopLevelItem(item)
        self.ui.eventsList.resizeColumnToContents(0)
        self.ui.eventsList.resizeColumnToContents(1)

    @Slot(QTreeWidgetItem, int)
    def eventDoubleClicked(self, item, col):
        event = item.data(0, Qt.UserRole)
        self.display_contact.emit(event.contact)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.capabilities.gallery import CapGallery, BaseGallery, BaseImage
from weboob.tools.json import json
from weboob.tools.backend import Module
from weboob.deprecated.browser import Browser, Page

__all__ = ['IzneoModule']


class ReaderV2(Page):
    def get_ean(self):
        return self.document.xpath("//div[@id='viewer']/attribute::rel")[0]

    def iter_gallery_images(self, gallery):
        ean = self.get_ean()
        pages = json.load(self.browser.openurl(
            'http://www.izneo.com/playerv2/ajax.php?ean=%s&action=get_list_jpg'
            % ean))

        for page in pages['list']:
            width = 1200  # maximum width
            yield BaseImage(page['page'],
                    gallery=gallery,
                    url=("http://www.izneo.com/playerv2/%s/%s/%s/%d/%s" %
                        (page['expires'], page['token'], ean, width, page['page'])))


class IzneoBrowser(Browser):
    PAGES = {r'http://.+\.izneo.\w+/readv2-.+': ReaderV2}

    def iter_gallery_images(self, gallery):
        self.location(gallery.url)
        assert self.is_on_page(ReaderV2)
        return self.page.iter_gallery_images(gallery)

    def fill_image(self, image, fields):
        if 'data' in fields:
            image.data = self.readurl(self.request_class(
                image.url, None, {'Referer': image.gallery.url}))


class IzneoModule(Module, CapGallery):
    NAME = 'izneo'
    MAINTAINER = u'Roger Philibert'
    EMAIL = 'roger.philibert@gmail.com'
    VERSION = '1.3'
    DESCRIPTION = 'Izneo digital comics'
    LICENSE = 'AGPLv3+'
    BROWSER = IzneoBrowser

    def iter_gallery_images(self, gallery):
        with self.browser:
            return self.browser.iter_gallery_images(gallery)

    def get_gallery(self, _id):
        match = re.match(r'(?:(?:.+izneo.com/)?readv2-)?(\d+-\d+)/?$', _id)
        if match is None:
            return None

        _id = match.group(1)

        gallery = BaseGallery(_id, url=('http://www.izneo.com/readv2-%s' % _id))
        with self.browser:
            return gallery

    def fill_gallery(self, gallery, fields):
        gallery.title = gallery.id

    def fill_image(self, image, fields):
        with self.browser:
            self.browser.fill_image(image, fields)

    OBJECTS = {
            BaseGallery: fill_gallery,
            BaseImage: fill_image}



# -*- coding: utf-8 -*-

# Copyright(C) 2011  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import PagesBrowser, URL

from .pages import ValidationPage, HomePage, HistoryPage, StoryPage, AuthorPage

# Browser


class HDSBrowser(PagesBrowser):
    BASEURL = 'http://histoires-de-sexe.net'

    validation_page = URL('^/$', ValidationPage)
    home = URL(r'/menu.php', HomePage)
    history = URL(r'/sexe/histoires-par-date.php\?p=(?P<pagenum>\d+)', HistoryPage)
    story = URL(r'/sexe.php\?histoire=(?P<id>.+)', StoryPage)
    author = URL(r'/fiche.php\?auteur=(?P<name>.+)', AuthorPage)

    def iter_stories(self):
        n = 1
        self.history.go(pagenum=n)
        while self.page.get_numerous() == n:
            for story in self.page.iter_stories():
                yield story

            n += 1
            self.history.go(pagenum=n)

    def get_story(self, id):
        self.story.go(id=id)

        assert self.story.is_here()
        return self.page.get_story()

    def get_author(self, name):
        self.author.go(name=name)

        assert self.author.is_here()
        return self.page.get_author()



# -*- coding: utf-8 -*-

# Copyright(C) 2012-2014 Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import calendar
from datetime import datetime
from decimal import Decimal

from weboob.browser.pages import HTMLPage, LoggedPage
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.filters.standard import Date, CleanText, Filter,\
    CleanDecimal, Regexp, Field, DateTime, Format, Env
from weboob.browser.filters.html import Attr
from weboob.capabilities.bill import Detail, Bill
from weboob.capabilities.base import NotAvailable
from weboob.exceptions import ParseError


class FormatDate(Filter):
    def filter(self, txt):
        return datetime.strptime(txt, "%Y%m%d").date()


class BadUTF8Page(HTMLPage):
    ENCODING = 'UTF-8'


class DetailsPage(LoggedPage, BadUTF8Page):
    def load_virtual(self, phonenumber):
        for div in self.doc.xpath('//div[@class="infosLigne pointer"]'):
            if CleanText('.')(div).split("-")[-1].strip() == phonenumber:
                return Attr('.', 'onclick')(div).split('(')[1][1]

    def on_load(self):
        self.details = {}
        for div in self.doc.xpath('//div[@class="infosConso"]'):
            num = div.attrib['id'].split('_')[1][0]
            self.details[num] = []

            # National parsing
            divnat = div.xpath('div[@class="national"]')
            if divnat:
                divnat = divnat[0]
                self._parse_div(divnat, "National : %s | International : %s", num, False)

            # International parsing
            divint = div.xpath('div[@class="international hide"]')
            if divint:
                divint = divint[0]
                if divint.xpath('div[@class="detail"]'):
                    self._parse_div(divint, u"Appels émis : %s | Appels reçus : %s", num, True)

    def _parse_div(self, divglobal, string, num, inter=False):
        divs = divglobal.xpath('div[@class="detail"]')
        # Two pieces of information in one div...
        div = divs.pop(0)
        voice = self._parse_voice(div, string, num, inter)
        self.details[num].append(voice)
        self._iter_divs(divs, num, inter)

    def _iter_divs(self, divs, num, inter=False):
        for div in divs:
            detail = Detail()
            detail.label = CleanText('div[@class="titre"]/p')(div)
            detail.id = "-" + detail.label.split(' ')[1].lower()
            if inter:
                detail.label = detail.label + u" (international)"
                detail.id = detail.id + "-inter"
            detail.infos = CleanText('div[@class="conso"]/p')(div)
            detail.price = CleanDecimal('div[@class="horsForfait"]/p/span', default=Decimal(0), replace_dots=True)(div)

            self.details[num].append(detail)

    def _parse_voice(self, div, string, num, inter=False):
        voicediv = div.xpath('div[@class="conso"]')[0]
        voice = Detail()
        voice.id = "-voice"
        voice.label = CleanText('div[@class="titre"]/p')(div)
        if inter:
            voice.label = voice.label + " (international)"
            voice.id = voice.id + "-inter"
        voice.price = CleanDecimal('div[@class="horsForfait"]/p/span', default=Decimal(0), replace_dots=True)(div)
        voice1 = CleanText('.//span[@class="actif"][1]')(voicediv)
        voice2 = CleanText('.//span[@class="actif"][2]')(voicediv)
        voice.infos = unicode(string) % (voice1, voice2)

        return voice

    # XXX
    def get_details(self, subscription):
        for detail in self.details[subscription._virtual]:
            detail.id = subscription.id + detail.id
            yield detail

    @method
    class date_bills(ListElement):
        item_xpath = '//div[has-class("factLigne")]'

        class item(ItemElement):
            klass = Bill

            def condition(self):
                num = Attr('.', 'data-fact_ligne', default='')(self)
                return self.env['subid'] == num

            obj_url = Attr('.//div[@class="pdf"]/a', 'href')
            obj__localid = Regexp(Field('url'), '&id=(.*)&date', u'\\1')
            obj_label = Regexp(Field('url'), '&date=(\d*)', u'\\1')
            obj_id = Format('%s.%s', Env('subid'), Field('_localid'))
            obj_date = FormatDate(Field('label'))
            obj_format = u"pdf"
            obj_type = u"bill"
            obj_price = CleanDecimal('div[@class="montant"]', default=Decimal(0), replace_dots=False)

    def get_renew_date(self, subscription):
        div = self.doc.xpath('//div[@login=$login]', login=subscription._login)[0]

        try:
            mydate = Date(CleanText('.//div[@class="resumeConso"]/span[@class="actif"][1]'), dayfirst=True)(div)
            if mydate.month == 12:
                mydate = mydate.replace(month=1)
                mydate = mydate.replace(year=mydate.year + 1)
            else:
                try:
                    mydate = mydate.replace(month=mydate.month + 1)
                except ValueError:
                    lastday = calendar.monthrange(mydate.year, mydate.month + 1)[1]
                    mydate = mydate.replace(month=mydate.month + 1, day=lastday)
            return mydate
        except ParseError:
            return NotAvailable

    def get_login(self, phonenumber):
        return Attr('.', 'login')(self.doc.xpath('//div[div[contains(text(), $phone)]]', phone=phonenumber)[0])

class HistoryPage(LoggedPage, BadUTF8Page):
    @method
    class get_calls(ListElement):
        item_xpath = '//tr'

        class item(ItemElement):
            klass = Detail

            def condition(self):
                txt = self.el.xpath('td[1]')[0].text
                return (txt is not None) and (txt != "Date")

            obj_id = None
            obj_datetime = DateTime(CleanText('td[1]', symbols=u'à'), dayfirst=True)
            obj_label = Format(u'%s %s %s', CleanText('td[2]'), CleanText('td[3]'),
                               CleanText('td[4]'))
            obj_price = CleanDecimal('td[5]', default=Decimal(0), replace_dots=True)



# -*- coding: utf-8 -*-

# Copyright(C) 2014 Vicnet
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import PagesBrowser, URL

from .pages import ListingAutoPage, AdvertPage

__all__ = ['LaCentraleBrowser']


class LaCentraleBrowser(PagesBrowser):
    BASEURL = 'http://www.lacentrale.fr'

    list_page = URL('/listing_auto.php\?(?P<_request>.*)',
                    ListingAutoPage)
    advert_page = URL('/auto-occasion-annonce-(?P<_id>.*).html', AdvertPage)

    def iter_prices(self, product):
        _request = '&'.join(['%s=%s' % (key, item) for key, item in product._criteria.iteritems()])
        return self.list_page.go(_request=_request).iter_prices()

    def get_price(self, _id, obj):
        return self.advert_page.go(_id=_id).get_price(obj=obj)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.test import BackendTest


class MeteoFranceTest(BackendTest):
    MODULE = 'meteofrance'

    def test_meteofrance(self):
        l = list(self.backend.iter_city_search('paris'))
        self.assertTrue(len(l) > 0)

        city = l[0]
        current = self.backend.get_current(city.id)
        self.assertTrue(current.temp.value > -20 and current.temp.value < 50)

        forecasts = list(self.backend.iter_forecast(city.id))
        self.assertTrue(len(forecasts) > 0)

        forecast2 = list(self.backend.iter_forecast('blagnac'))
        self.assertTrue(len(forecast2) > 0)



# -*- coding: utf-8 -*-

# Copyright(C) 2012 Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re
import os
import subprocess
import tempfile
import shutil

from datetime import datetime, date, time
from decimal import Decimal

from weboob.deprecated.browser import Page
from weboob.capabilities.bill import Detail, Bill


def _get_date(detail):
    return detail.datetime


class PdfPage():
    def __init__(self, file):
        self.pdf = file

    def _parse_pdf(self):
        pdffile = tempfile.NamedTemporaryFile(bufsize=100000, mode='w', suffix='.pdf')
        temptxt = pdffile.name.replace('.pdf', '.txt')
        cmd = "ebook-convert"
        stdout = open("/dev/null", "w")
        shutil.copyfileobj(self.pdf, pdffile)
        pdffile.flush()
        subprocess.call([cmd, pdffile.name, temptxt], stdout=stdout)
        pdffile.close()
        txtfile = open(temptxt, 'r')
        txt = txtfile.read()
        txtfile.close()
        os.remove(temptxt)
        return txt

    def get_details(self):
        txt = self._parse_pdf()
        page = txt.split('CONSOMMATION')[2].split('ACTIVITE DETAILLEE')[0]
        lines = page.split('\n')
        lines = [x for x in lines if len(x) > 0]  # Remove empty lines
        details = []
        detail = None
        lines.pop(-1)  # Line to describes pictures
        twolines = False
        for line in lines:
            if "Votre consommation" in line:
                line = line.split(": ", 1)[1]
            if twolines:
                twolines = False
                detail.infos = unicode(line, encoding='utf-8')
            elif re.match('[A-Za-z]', line[0]):
                # We have a new element, return the other one
                if detail is not None:
                    details.append(detail)
                detail = Detail()
                split = re.split("(\d)", line, maxsplit=1)
                detail.price = Decimal(0)
                if len(split) > 2:
                    detail.infos = unicode(split[1] + split[2], encoding='utf-8')
                else:
                    twolines = True
                if '€' in line:
                    specialprice = split[1] + split[2]
                    detail.price = Decimal(specialprice.replace('€', ''))
                detail.label = unicode(split[0], encoding='utf-8')
            elif '€' in line:
                detail.price = Decimal(line.replace('€', ''))
            else:
                detail.infos = unicode(line, encoding='utf-8')
        details.append(detail)
        return details

    # Standard pdf text extractor take text line by line
    # But the position in the file is not always the "real" position to display...
    # It produce some unsorted and unparsable data
    # Example of bad software: pdfminer and others python tools
    # This is why we have to use "ebook-convert" from calibre software,
    # it is the only one to 'reflow" text and give some relevant results
    # The bad new is that ebook-convert doesn't support simple use with stdin/stdout
    def get_calls(self):
        txt = self._parse_pdf()
        pages = txt.split("DEBIT")
        pages.pop(0)  # remove headers
        details = []
        for page in pages:
            page = page.split('RÉGLO MOBILE')[0].split('N.B. Prévoir')[0]  # remove footers
            lines = page.split('\n')
            lines = [x for x in lines if len(x) > 0]  # Remove empty lines
            numitems = (len(lines) + 1) / 4  # Each line has five columns
            lines.pop(0)  # remove the extra € symbol
            modif = 0
            i = 0
            while i < numitems:
                if modif != 0:
                    numitems = ((len(lines) + 1 + modif) / 4)
                base = i * 4 - modif
                dateop = base
                corres = base + 1
                duree = base + 2
                price = base + 3
                if "Changement vers le Forfait" in lines[base]:
                    modif += 1
                    i += 1
                    continue
                # Special case with 5 columns, the operation date is not in the first one
                if len(re.split("(\d+\/\d+\/\d+)", lines[dateop])) < 2:
                    lines[base + 1] = lines[base] + " " + lines[base + 1]
                    dateop = base + 1
                    corres = base + 2
                    duree = base + 3
                    price = base + 4
                    modif -= 1
                detail = Detail()
                splits = re.split("(\d+\/\d+\/\d+)", lines[dateop])
                mydate = date(*reversed([int(x) for x in splits[1].split("/")]))
                mytime = time(*[int(x) for x in splits[2].split(":")])
                detail.datetime = datetime.combine(mydate, mytime)
                if lines[corres] == '-':
                    lines[corres] = ""
                if lines[duree] == '-':
                    lines[duree] = ''
                detail.label = unicode(splits[0], encoding='utf-8', errors='replace') + u" " + lines[corres] + u" " + lines[duree]
                # Special case with only 3 columns, we insert a price
                if "Activation de votre ligne" in detail.label or u"Résiliation" in detail.label:
                    lines.insert(price, '0')
                try:
                    detail.price = Decimal(lines[price].replace(',', '.'))
                except:
                    # In some special cases, there are no price column. Try to detect it
                    if "Inclus" not in lines[price]:
                        modif += 1
                    detail.price = Decimal(0)

                details.append(detail)
                i += 1
        return sorted(details, key=_get_date, reverse=True)


class HistoryPage(Page):
    def on_loaded(self):
        pass

    def getmaxid(self):
        max = 1
        while len(self.document.xpath('//li[@id="liMois%s"]' % max)) > 0:
            max += 1
        return max - 1

    def date_bills(self, parentid):
        max = 1
        while len(self.document.xpath('//li[@id="liMois%s"]' % max)) > 0:
            li = self.document.xpath('//li[@id="liMois%s"]' % max)[0]
            max += 1
            link = li.xpath('a')[0]
            bill = Bill()
            bill.url = unicode(link.attrib['href'])
            bill.label = unicode(link.text)
            bill.format = u"pdf"
            bill.type = u"bill"
            bill.id = parentid + bill.label.replace(' ', '')
            yield bill



# -*- coding: utf-8 -*-

# Copyright(C) 2016      François Revol
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import PagesBrowser, URL
from weboob.tools.compat import quote_plus

from .pages import SearchPage, AdvertPage


class LinuxJobsBrowser(PagesBrowser):
    BASEURL = 'https://www.linuxjobs.fr'

    advert_page = URL('/jobs/(?P<id>.+)', AdvertPage)
    search_page = URL('/search/(?P<job>)', SearchPage)

    def get_job_advert(self, _id, advert):
        self.advert_page.go(id=_id)

        assert self.advert_page.is_here()
        return self.page.get_job_advert(obj=advert)

    def search_job(self, pattern=None):
        if pattern is None:
            return []
        self.search_page.go(job=quote_plus(pattern.encode('utf-8')))

        assert self.search_page.is_here()
        return self.page.iter_job_adverts()



# -*- coding: utf-8 -*-

# Copyright(C) 2017      Phyks (Lucas Verney)
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals


from weboob.browser import LoginBrowser, URL, need_login
from weboob.browser.exceptions import ClientError
from weboob.exceptions import BrowserIncorrectPassword

from .pages import LoginPage, BillsPage


class LampirisBrowser(LoginBrowser):
    BASEURL = 'https://espaceclient.lampiris.fr/'

    loginpage = URL('/user/login', LoginPage)
    billspage = URL('/factures-et-paiements', BillsPage)
    selectcus = URL('/set_selected_cus')

    def do_login(self):
        self.loginpage.stay_or_go().do_login(self.username, self.password)

        try:
            self.billspage.go()
        except ClientError:
            raise BrowserIncorrectPassword()

    @need_login
    def get_subscriptions(self):
        return self.billspage.go().get_subscriptions()

    @need_login
    def get_documents(self, subscription):
        # Select subscription
        self.selectcus.go(params={'cus': subscription})

        # Then, fetch documents
        return self.billspage.go().get_documents()



from weboob.capabilities.housing import Query

QUERY_TYPES = {
    Query.TYPE_RENT: 'location',
    Query.TYPE_SALE: 'achat'
}

QUERY_HOUSE_TYPES = {
    Query.HOUSE_TYPES.APART: ['appartement'],
    Query.HOUSE_TYPES.HOUSE: ['maison'],
    Query.HOUSE_TYPES.PARKING: ['parking'],
    Query.HOUSE_TYPES.LAND: ['terrain'],
    Query.HOUSE_TYPES.OTHER: ['chambre', 'appartement-meuble',
                              'local-commercial', 'immeuble']
}

AVAILABLE_TYPES = {
    Query.TYPE_RENT: ['appartement', 'maison', 'parking', 'chambre',
                      'appartement-meuble', 'local-commercial'],
    Query.TYPE_SALE: ['appartement', 'maison', 'parking', 'local-commercial',
                      'terrain', 'immeuble']
}
